/* Copyright (c) 2001 Michael A Wolf.  All rights reserved.

This file is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the 
Free Software Foundation; either version 2, or (at your option) any later
version.

This file is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along
with this file; see the file COPYING.  If not, write to the 
Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

/* $Header: /cvsroot/moodns/moodns/code/sockcat/xmalloc.c,v 1.1 2002/07/20 16:46:58 mawolf Exp $  */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "xmalloc.h"

void *
wmalloc(size_t size, const char *file, int line)
{
    void *retval;

    retval = malloc(size);
    if (retval == NULL) {
        fprintf(stderr, "file %s, line %d: malloc() failed\n", file, line);
        fprintf(stderr, "(attempted to allocate %d bytes)\n", size);
        abort();
    }

#ifdef WALLOC_NASTY
    {
        int i;

        srand(time());
        for (i = 0; i < size; ++i) {
            if (i % 2 == 0)
                ((int *) retval)[i] = 0xDEADBEEF;
            else 
                ((int *) retval)[i] = rand();
        }
    }
#endif

    return (retval);    
}


void *
wcalloc(size_t nmemb, size_t size, const char *file, int line)
{
    void *retval;

    retval = calloc(nmemb, size);
    /* retval = NULL; */
    if (retval == NULL) {
        fprintf(stderr, "file %s, line %d: calloc() failed\n", file, line);
        fprintf(stderr, "(attempted to allocate %d * %d = %d bytes)\n", nmemb,
                size, nmemb * size);
        abort();
    }
    return (retval);
}

void *
wrealloc(void *p, size_t size, const char *file, int line)
{
    void *retval;

    retval = NULL;
    retval = realloc(p, size);
    if (retval == NULL) {
        fprintf(stderr, "file %s, line %d: realloc() failed\n", file, line);
        fprintf(stderr, "(attempted to allocate %d bytes)\n", size);
        abort();
    }

    return (retval);
}

void 
wfree(void *p, const char *file, int line)
{
    if (p == NULL) {
        fprintf(stderr, "wfree() called on a NULL pointer: file %s, line %d\n",
                file, line);
        fprintf(stderr, "technically this is legal, but it's a dubious \n");
        fprintf(stderr, "practise.  hence this warning.\n");
    }
    free(p);
}

char *
wstrdup(const char *p, const char *file, int line)
{
    char *retval;
    retval = strdup(p);
    if (retval == NULL) {
        fprintf(stderr, "file %s, line %d: strdup() failed\n", file, line);
        fprintf(stderr, "(attempted to allocate %d bytes)\n", strlen(p));
        abort();
    }
    return (retval);
}



