/* 
   Copyright (C) 2001 Michael Wolf (mike@dodgysoftware.net)
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:
   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
   3. The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.
 
   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
   IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
   TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/




/* This file is based on an implementation of fgetln() distributed with 
   lukemftp, by Luke Mewburn.  The original implementation and copyright
   notice are included at the bottom of the file.  */

/* $Header: /cvsroot/moodns/moodns/src/afgetln.c,v 1.4 2002/02/01 06:15:15 mawolf Exp $  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "misc.h"
#include "xmalloc.h"
#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

/* Probably a good idea to do something with real checking.  */
/* #define xmalloc malloc
 #define xcalloc calloc
#define xrealloc realloc
#define xfree free  */


#define BUFCHUNKS   1024

char *
afgetln(FILE *fp)
{
    char *retval;
    size_t bufsize;
    size_t buflen;
    char curbuf[BUFCHUNKS];
    char *p;

    bufsize = BUFCHUNKS;
    retval = (char *) xcalloc(bufsize, sizeof(char));
    buflen = 0;
    while ((p = fgets(curbuf, sizeof(curbuf), fp)) != NULL) {
        size_t l;
        l = strlen(p);
        if (bufsize < buflen + l) {
            bufsize += BUFCHUNKS;
            retval = (char *) xrealloc(retval, bufsize);
        }
        strcpy(retval + buflen, p);
        buflen += l;
        if (p[l - 1] == '\n') {
            p[l - 1] = '\0';
            break;
        }
    }
    if (p == NULL && *retval == '\0') {
        xfree(retval);
        retval = NULL;
    } else {
        p = strchr(retval, '\n');
        if (p != NULL) 
            *p = '\0';  /* hmmm...  */
    }

    return (retval);
}






#ifdef TEST

int
main(int argc, char **argv)
{
    FILE *n;
    char *x; 
    int l, c; 
    int ret; 

#ifndef WITH_DMALLOC
    extern char *malloc_options;
    malloc_options = "Z";
#endif

    x = NULL;
    l = c = 0;

    n = fopen("COPYING", "r");
    if (n == NULL) {
        perror("unable to open the file");
        abort();
    }
    while (!feof(n)) {
        x = afgetln(n);
        if (x != NULL) {
            printf("(%d) we got %d bytes that time\n", ++c, strlen(x));
            free(x);
        }
    }
    if ((ret = fclose(n)) != 0) {
        perror("fclose() failed");
        abort();
    }

    return 0;
}

#endif






/*
 * Copyright 1999 Luke Mewburn <lukem@netbsd.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#if 0

char *
fgetln(FILE *fp, size_t *len)
{
    static char *buf;
    static size_t bufsize;
    size_t buflen;
    char curbuf[BUFCHUNKS];
    char *p;
     
    if (buf == NULL) {
        bufsize = BUFCHUNKS;
        buf = (char *) xmalloc(bufsize);
    }
        
    *buf = '\0';
    buflen = 0;
    while ((p = fgets(curbuf, sizeof(curbuf), fp)) != NULL) {
        size_t l;

        l = strlen(p);
        if (bufsize < buflen + l) {
            bufsize += BUFCHUNKS;
            buf = (char *) xrealloc(buf, bufsize);
        }
        strcpy(buf + buflen, p);
        buflen += l;
        if (p[l - 1] == '\n')
            break;
    }
    if (p == NULL && *buf == '\0')
        return (NULL);
    *len = strlen(buf);
    return (buf);
}

#endif

