/* Copyright (c) 2001, 2002 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 * 
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 * 
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/* $Header: /cvsroot/moodns/moodns/src/auth_data.c,v 1.12 2002/02/21 05:09:45 mawolf Exp $  */


/* Uses a format similar to that of djbdns' tinydns.  Maybe even the same.  
   Based on documentation presented at 
   http://cr.yp.to/djbdns/tinydns-data.html.  */

/* TODO: Finish.  Also: break down into functions; it's getting a bit
   messy.  */

/* FIXME: implement the ip prefix crapola  */

/* FIXME: the IP address from @ lines is coming up as 0.  Fix.  */

#if HAVE_CONFIG_H
# include "../config.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>


#include "misc.h"
#include "options.h"
#include "record.h"
#include "ht.h"
#include "auth_data.h"
#include "xmalloc.h"
#if WITH_DMALLOC
# include <dmalloc.h>
#endif

extern struct global_options globals;

void _break(char *, int *);
void _strip(char *, int *);
void _ugly_split(char *, char, int *, char ***);

struct record *_record_basic_init(enum record_type);
int _warning_range(int, int, int, int);
unsigned long long _do_timestamp(const char *);
unsigned long _do_ttl(const char *);

void _do_dot(struct record **, struct record **, struct record **, int, char **, int);
void _do_ampersand(struct record **, struct record **, int, char **, int);
void _do_equals(struct record **, struct record **, int, char **, int);
void _do_plus(struct record **, int, char **, int);
void _do_at(struct record **, struct record **, int, char **, int);
void _do_txt(struct record **, int, char **, int);
void _ip_prefix_warning(void);
void _unimplemented_warning(int);

enum {
    MAX_LABEL_SIZE = 255
} big_constants;

/* FIXME: This function is going to need to talk to something that stores
   data.  As mentioned elsewhere, I think, I want to be able to store data
   in several formats: memory, sleepycat db, gdbm, cdb, possibly also
   hybrid memory/disk cache schemes.  I think this function needs to be passed
   a function pointer and a piece or two of auxiliary data - when a record
   is created, it is passed to the function along w/ the aux data.  */

/* FIXME: implement the "uncommon data lines" too.  */

/* FIXME: Replace instances of "255" and "256" with proper symbols.  */

int 
load_auth_data(char *path, int (*func)(struct record *, void *), void *dp)
{
    int retval;
    FILE *f;

    f = fopen(path, "r");
    if (f == NULL) 
        retval = 1;
    else
        retval = load_auth_data_f(f, func, dp);

    return retval;
}

int 
load_auth_data_f(FILE *f, int (*func)(struct record *, void *), void *dp)
{
    int retval;
    char *line, *line2;
    int pos;
    struct record *rec1, *rec2, *rec3;
    char **fields; 
    int num_fields;
    int line_no;
    
    line_no = 1;    
    while ((line = afgetln(f)) != NULL) {
        line2 = &line[1];
        _strip(line2, &pos);
        _ugly_split(line2, ':', &num_fields, &fields);        

        rec1 = rec2 = rec3 = NULL;
        
        switch (*line) {
        case '.':
            _do_dot(&rec1, &rec2, &rec3, num_fields, fields, line_no);
            break;
        case '&': 
            _do_ampersand(&rec1, &rec2, num_fields, fields, line_no);
            break;
        case '=':
            _do_equals(&rec1, &rec2, num_fields, fields, line_no);
            break;
        case '+': 
            _do_plus(&rec1, num_fields, fields, line_no);
            break;
        case '@':
            _do_at(&rec1, &rec2, num_fields, fields, line_no);
            break;
        case '\'':
            _do_txt(&rec1, num_fields, fields, line_no);
            break;
        case '^':
            printf("^ - PTR record\n");
            _unimplemented_warning(line_no);
            break;
        case 'C':
            printf("C - CNAME record\n");
            _unimplemented_warning(line_no);
            break;
        case 'Z':
            printf("SOA record\n");
            _unimplemented_warning(line_no);
            break;
        case ':':
            printf("Generic record.  Hrm.\n");
            _unimplemented_warning(line_no);
            break;
        case '-':
            printf("ignored\n");
            _unimplemented_warning(line_no);
            break;
        case '%':
            printf("ip prefix\n");
            break;
        default:
            printf("unknown\n");
            _unimplemented_warning(line_no);
            break;
        }
        
        free(line);
        ++line_no;

        {
            /* FIXME: This is a bunch of crap.  I think.  */
            int i;
            if (rec1) {
                func(rec1, dp);
                xfree(rec1);
            }
            if (rec2) {
                func(rec2, dp);
                xfree(rec2);
            }
            if (rec3) {
                func(rec3, dp);
                xfree(rec3);
            }
            for (i = 0; i < num_fields; ++i) {
                xfree(fields[i]);
            }
            xfree(fields);
        }
    }
    fclose(f);
    retval = 0;

    return (retval);
}

void
_break(char *str, int *no_colons)
{
    int len, i;

    assert(str != NULL);
    *no_colons = 0;
    len = strlen(str);
    for (i = 0; i < len; ++i) {
        if (str[i] == ':') {
            str[i] = '\0';
            ++*no_colons;
        }
    }
}

/* Get rid of comments and trailing whitespace.  */
void
_strip(char *str, int *pos)
{
    char *c;

    *pos = -1;
    for (c = str; *c != '\0'; ++c) {
        if (*c == '#' || isspace(*c)) {
            *pos = c - str;
            break;
        }
    }
}

void
_ugly_split(char *original_string, char delim, int *num_fields, 
            char ***first_ptr)
{
    char *new_string;
    char **other_stuff;
    int n;
    char *x;
    
    n = strlen(original_string);
    new_string = (char *) alloca(n + 1);
    strcpy(new_string, original_string);

    other_stuff = NULL;

    *num_fields = 0;

    while ((x = strchr(new_string, delim)) != NULL) {
        *x = '\0';
        other_stuff = (char **) xrealloc(other_stuff, ++*num_fields * sizeof(char *));
        other_stuff[*num_fields - 1] = xstrdup(new_string);
        new_string = x + 1;
    }
    other_stuff = (char **) xrealloc(other_stuff, ++*num_fields * sizeof(char *));
    other_stuff[*num_fields - 1] = xstrdup(new_string);
    
    *first_ptr = other_stuff;
}

struct record *
_record_basic_init(enum record_type rt)
{
    struct record *retval;
    retval = (struct record *) xcalloc(1, sizeof(struct record));
    retval->type = rt;
    retval->ttl = globals.authoritative.default_ttl;
    
    return (retval);
}

int
_warning_range(int actual, int low, int high, int line)
{
    int retval;
    
    if (actual < low) {
        printf("Warning: too few fields in line %d\n", line);
        retval = -1;
    } else if (actual > high) {
        printf("Warning: too many fields in line %d\n", line);
        retval = -2;
    } else {
        retval = 0;
    }
    return (retval);
}

unsigned long long
_do_timestamp(const char *whenever)
{
    unsigned long long retval;
    /* Using djb's tai library doesn't seem to be necessary.  */
    sscanf(whenever, "%qx", &retval);  
    /* FIXME: we should check the return value - if the conversion
       failed, invent a timestamp.  */
    return (retval);
}

unsigned long
_do_ttl(const char *howlong) 
{
    unsigned long retval;
    char *np;
    /* FIXME: we should check for something wrt np - if strtoul
       failed, don't set the ttls.  */

    if (strlen(howlong) > 0) 
        retval = strtoul(howlong, &np, 10);
    else
        retval = globals.authoritative.default_ttl;

    return (retval);
}

void
_do_dot(struct record **rec1, struct record **rec2, struct record **rec3,
        int num_fields, char **fields, int line_no)
{
    unsigned long long ts;
    unsigned long ttl;
    char *name;
    size_t len;
    char *tmp;

    /* printf(". - NS, A, and SOA records\n");  */
    if (_warning_range(num_fields, 2, 5, line_no) < 0)
        return;
    
    *rec1 = _record_basic_init(A);
    *rec2 = _record_basic_init(NS);
    *rec3 = _record_basic_init(SOA);
    switch (num_fields) {
    case 6:
        _ip_prefix_warning();
    case 5:
        ts = _do_timestamp(fields[4]);
        (*rec1)->timestamp = ts;
        (*rec2)->timestamp = ts;
        (*rec3)->timestamp = ts;
    case 4:
        ttl = _do_ttl(fields[3]);
        (*rec1)->ttl = ttl;
        (*rec2)->ttl = ttl;
        (*rec3)->ttl = ttl;
    case 3:
        name = fields[2];
        if (strchr(name, '.') != NULL) {
            /* Absolute name.  */
            if (name[strlen(name) - 1] == '.') 
                name[strlen(name) - 1] = '\0';
            strncpy((*rec2)->record.ns, name, 255);
        } else {
            /* Name appended: name.ns.fqdn  */
            len = strlen(name) + strlen(".ns.")
                + strlen(fields[0]) + 1;
            tmp = (char *) alloca(len * sizeof(char));
            memset(tmp, '\0', len);
            strcat(tmp, name);
            strcat(tmp, ".ns.");
            strcat(tmp, fields[0]);
            strncpy((*rec2)->record.ns, tmp, 255);
        }
    case 2:
        if (strlen(fields[1]) > 0) {
            (*rec1)->record.a = inet_addr(fields[1]);
        }
    case 1:   
        strncpy((*rec2)->record.ns, fields[0], 255);
        strncpy((*rec3)->record.soa.mname, fields[0], 255);
        strncpy((*rec3)->record.soa.rname, "hostmaster", 255);
        /* Hmm.  */
        (*rec3)->record.soa.serial = 0;
        (*rec3)->record.soa.refresh = 0;
        (*rec3)->record.soa.retry = 0;
        (*rec3)->record.soa.expire = 0;
        strncpy((*rec1)->name, fields[0], 255);
        strncpy((*rec2)->name, fields[0], 255);        
        strncpy((*rec3)->name, fields[0], 255);        
    }
}

void
_do_ampersand(struct record **rec1, struct record **rec2, int num_fields,
              char **fields, int line_no)
{
    unsigned long long ts;
    unsigned long ttl;
    char *name;
    size_t len;
    char *tmp;

    /* printf("& - NS and A records\n");  */
    if (_warning_range(num_fields, 2, 5, line_no) < 0)
        return;
    *rec1 = _record_basic_init(A);
    *rec2 = _record_basic_init(NS);
    switch (num_fields) {
    case 6:
        _ip_prefix_warning();
    case 5:
        ts = _do_timestamp(fields[4]);
        (*rec1)->timestamp = ts;
        (*rec2)->timestamp = ts;
    case 4:
        ttl = _do_ttl(fields[3]);
        (*rec1)->ttl = ttl;
        (*rec2)->ttl = ttl;
    case 3:
        name = fields[2];
        if (strchr(name, '.') != NULL) {
            if (name[strlen(name) - 1] == '.')
                name[strlen(name) - 1] = '\0';
            strncpy((*rec2)->record.ns, name, 255);
        } else {
            len = strlen(name) + strlen(".ns.")
                + strlen(fields[0]) + 1;
            tmp = (char *) alloca(len * sizeof(char));
            memset(tmp, '\0', len);
            strcat(tmp, name);
            strcat(tmp, ".ns.");
            strcat(tmp, fields[0]);
            strncpy((*rec2)->record.ns, tmp, 255);
        }
    case 2:
        if (strlen(fields[1]) > 0) {
            (*rec1)->record.a = inet_addr(fields[1]);
        }
    case 1:
        strncpy((*rec2)->record.ns, fields[0], 255);
        strncpy((*rec1)->name, fields[0], 255);
        strncpy((*rec2)->name, fields[0], 255);
    }
}

void
_do_equals(struct record **rec1, struct record **rec2, int num_fields, 
           char **fields, int line)
{
    unsigned long long ts;
    unsigned long ttl;

    /* printf("= - A record and a reverse PTR record\n");  */
    if (_warning_range(num_fields, 2, 5, line) < 0)
        return;
    *rec1 = _record_basic_init(A);
    *rec2 = _record_basic_init(PTR);
    switch (num_fields) {
    case 5:
        _ip_prefix_warning();
    case 4:
        ts = _do_timestamp(fields[2]);
        (*rec1)->timestamp = ts;
        (*rec2)->timestamp = ts;
    case 3:
        ttl = _do_ttl(fields[2]);
        (*rec1)->ttl = ttl;
        (*rec2)->ttl = ttl;
    case 2:
        (*rec1)->record.a = inet_addr(fields[1]);
        /* FIXME: What do we do here?  */
    case 1:
        strncpy((*rec1)->name, fields[0], 255);
        strncpy((*rec2)->name, fields[0], 255);
        /* FIXME: What do we do here?  */
    }
    
}

void
_do_plus(struct record **rec1, int num_fields, char **fields, int line __attribute__((unused)))
{
    unsigned long long ts;
    unsigned long ttl;

    /* printf("+ - A record (no reverse PTR record)\n");  */
    *rec1 = _record_basic_init(A);
    switch (num_fields) {
    case 5:
        _ip_prefix_warning();
    case 4:
        ts = _do_timestamp(fields[3]);
        (*rec1)->timestamp = ts;
    case 3:
        ttl = _do_ttl(fields[2]);
        (*rec1)->ttl = ttl;
    case 2:
        (*rec1)->record.a = inet_addr(fields[1]);
    case 1:
        strncpy((*rec1)->name, fields[0], 255);
    }
}

void
_do_at(struct record **rec1, struct record **rec2, int num_fields, 
        char **fields, int line_no __attribute__((unused)))
{
    unsigned long long ts;
    unsigned long ttl;
    u16 preference;
    char *np;
    char *name;
    char *mx_name;
    size_t len;
    char *tmp;
    
    /* FIXME: finish this.  */
    /* printf("@ - MX record and A record (to foo.mx.fqdn)\n");  */

    (*rec1) = (struct record *) xcalloc(1, sizeof(struct record));
    (*rec2) = (struct record *) xcalloc(1, sizeof(struct record));
    
    (*rec1)->type = MX;
    (*rec2)->type = A;
    
    switch (num_fields) {
    case 7:
        _ip_prefix_warning();
    case 6:
        ts = _do_timestamp(fields[5]);
        (*rec1)->timestamp = ts;
        (*rec2)->timestamp = ts;
    case 5:
        ttl = _do_ttl(fields[4]);
        (*rec1)->ttl = ttl;
        (*rec2)->ttl = ttl;
    case 4:
        preference = (u16) strtoul(fields[3], &np, 10); /* Well, this is dangerous.  */
        (*rec1)->record.mx.preference = preference;
    case 3:
        /* "If x contains a dot then it is treated differently; see
           above." (tinydns-data.html).  I assume this means mx records
           get treatment analogous to what ns records get.  Bleh.  */
        name = fields[2];
        if (strchr(name, '.') != NULL) {
            if (name[strlen(name) - 1] == '.') 
                name[strlen(name) - 1] = '\0';
            strncpy((*rec1)->record.mx.exchange, name, 255);
            mx_name = name;
        } else {
            len = strlen(name) + strlen(".ns.") + strlen(fields[0]) + 1;
            tmp = alloca(len * sizeof(char));
            memset(tmp, '\0', len);
            strcat(tmp, name);
            strcat(tmp, ".mx.");
            strcat(tmp, fields[0]);
            strncpy((*rec1)->record.mx.exchange, tmp, 255);
        }
    case 2:
        if (strlen(fields[1]) > 0) {
            (*rec2)->record.a = inet_addr(fields[1]);
        }
    case 1:
        strncpy((*rec1)->name, fields[0], 255);
        strncpy((*rec2)->name, fields[0], 255);
    }
}

void
_do_txt(struct record **rec1, int num_fields, char **fields, 
        int line_no __attribute__((unused)))
{
    unsigned long long ts;
    unsigned long ttl;

    /* printf("' - TXT record\n");  */

    (*rec1) = (struct record *) xcalloc(1, sizeof(struct record));
    (*rec1)->type = TXT;

    switch (num_fields) {
    case 5:
        _ip_prefix_warning();
    case 4:
        ts = _do_timestamp(fields[3]);
        (*rec1)->timestamp = ts;
    case 3:
        ttl = _do_ttl(fields[2]);
        (*rec1)->ttl = ttl;
    case 2:
        strncpy((*rec1)->record.txt, fields[1], 255);
    case 1:
        strncpy((*rec1)->name, fields[0], 255);
    }
}

void
_ip_prefix_warning()
{
    printf("Warning: ip prefixes not yet implemented.\n");
}

void
_unimplemented_warning(int lineno)
{
    printf("Warning: unimplemented feature requested, line %d\n", lineno);
}

#ifdef TEST

/* We currently can build using gdbm, db4, , or a default of ht.  */

#include <fcntl.h>
#include <sys/stat.h>

char *malloc_options = "AJR";

struct global_options globals;

#if defined(WITH_GDBM)
/* Play with gdbm  */
#include <gdbm.h>
static void _gdbm_fatal(char *);
static int _gdbm_very_simple(struct record *, void *);
#elif defined(WITH_DB4)
#include <db.h>
static int _db4_very_simple(struct record *, void *);

#elif defined(WITH_DB3)

#else
/* Play with ht  */
int very_simple(struct record *, void *);
int _strcmp_local(void *, void *, void *);
void _free_local(void *, void *);
void strprint(void *);
#endif

int
main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
{
    int ret;

#if defined(WITH_GDBM)
    GDBM_FILE gdbm;

    memset(&globals, 0, sizeof(globals));
    gdbm = gdbm_open("data.gdbm", 0, GDBM_WRCREAT, S_IRUSR | S_IWUSR, 
                     _gdbm_fatal);
    load_auth_data("../doc/data", _gdbm_very_simple, &gdbm);

    gdbm_close(gdbm);
#elif defined(WITH_DB4)
    DB *dbp;
#define DATABASE "data.db"
    ret = db_create(&dbp, NULL, 0);
    ret = dbp->open(dbp, DATABASE, NULL, DB_HASH, DB_CREATE, 0644);

    load_auth_data("../doc/data", _db4_very_simple, dbp);
#else
    ht *xyz;
    struct record_chain *ch;

    memset(&globals, 0, sizeof(globals));
    xyz = ht_init(_strcmp_local, _free_local);
    load_auth_data("../doc/data", very_simple, xyz);
    ch = ht_get(xyz, "doobar.com");
    printf("well\n");
#endif

    return (0);
}

int
very_simple(struct record *rec, void *_abc)
{
    struct record_chain *ch;
    
    ht *abc = (ht *) _abc;
    /* printf("This is something.  We have a record of size %d.\n", sizeof(*rec));
       printf("rec->name is %s\n", rec->name);  */
    
    ch = ht_get(abc, rec->name);
    if (ch == NULL) {
        /* printf("it's not found\n");  */
        ch = record_chain_init(rec->name);
        record_chain_add(ch, rec);
        ht_put(abc, rec->name, ch, NULL);
    } else {
        /* printf("it's there already\n");  */
    }

    return 0;
}

int
_strcmp_local(void *_a, void *_b, void *_c __attribute__((unused)))
{
    return (strcmp((char *) _a, (char *) _b));
}

void
_free_local(void *_a, void *_b __attribute__((unused)))
{
    xfree(_a);
}

void
strprint(void *_arg)
{
    char *arg;
    arg = (char *) _arg;
    printf("%s", arg);
}



#if defined(WITH_GDBM)

void
_gdbm_fatal(char *msg)
{
    printf("gdbm is diedses: %s\n", msg);
    printf("error is: %s\n", gdbm_strerror(gdbm_errno));
}

int
_gdbm_very_simple(struct record *rec, void *_abc)
{

    struct record *tmp;
    struct record_chain *ch;
    
    GDBM_FILE *abc;
    
    datum key;
    datum data, data2;

    int ret;    

    abc = (GDBM_FILE *) _abc;
    
    key.dptr = rec->name;
    key.dsize = strlen(rec->name);

    RECORD_CHAIN_INIT_TMP(ch, rec->name);

    if (gdbm_exists(*abc, key) != 0) {
        printf("it's already there\n");

        data = gdbm_fetch(*abc, key);
        printf("We got a record chain of size %d\n", data.dsize);
        ch->records = (struct record *) data.dptr;
        ch->records_len = data.dsize / sizeof(struct record);
        

        record_chain_add(ch, rec);
        printf("There should now be %d records!\n", ch->records_len);
        
        data2.dptr = (char *) ch->records;
        data2.dsize = ch->records_len * sizeof(struct record);

        ret = gdbm_store(*abc, key, data2, GDBM_REPLACE);
    } else {
        printf("it's not there, so let's add it\n"); 

        record_chain_add(ch, rec);
        data.dptr = (char *) ch->records;
        data.dsize = ch->records_len * sizeof(struct record);
        
        ret = gdbm_store(*abc, key, data, GDBM_INSERT);
    }


    return 0;
}

#endif

#if defined(WITH_DB4)



int
_db4_very_simple(struct record *rec, void *_abc)
{
    struct record_chain *ch;
    DB *dbp = (DB *) _abc;

    DBT key, data, data2;
    int ret;
    

    RECORD_CHAIN_INIT_TMP(ch, rec->name);
   
    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
    memset(&data2, 0, sizeof(data));
    key.data = rec->name;
    key.size = strlen(rec->name);
    
    if ((ret = dbp->get(dbp, NULL, &key, &data, 0)) == 0) {
        printf("it exists\n");
        ch->records = (struct record *) data.data;
        ch->records_len = data.size / sizeof(struct record);

        record_chain_add(ch, rec);
        
        data2.data = ch->records;
        data2.size = ch->records_len * sizeof(struct record);

        ret = dbp->put(dbp, NULL, &key, &data2, 0);
    } else {
        printf("it's not there, so let's add it\n");
        record_chain_add(ch, rec);
        data.data = ch->records;
        data.size = ch->records_len * sizeof(struct record);
        ret = dbp->put(dbp, NULL, &key, &data, 0);
    }

    return 0;
}

#endif

#endif
