/* Copyright (c) 2002 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 * 
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 * 
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/* $Header: /cvsroot/moodns/moodns/src/database-gdbm.c,v 1.2 2002/01/27 10:19:17 mawolf Exp $  */

#ifndef _REENTRANT
#define _REENTRANT 
#endif

#include <gdbm.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include "record.h"
#include "database.h"

#include "xmalloc.h"
#if WITH_DMALLOC
# include <dmalloc.h>
#endif



#ifndef HAVE_GDBM


/* If you do not check the return value, then you will DIE!  */
int
database_module_init(database *d)
{
    int retval = 1;
    xfree(d);
    return (retval);    
}

#endif

/* Limbo.  */

#if defined(HAVE_GDBM)

#define CAST(x) ((struct ht_handle *) x)

struct gdbm_handle {
    FILE *f;
    GDBM_FILE *table;
};




// struct database_generic *get_handle(void);

static enum database_type _this_type(void);
static int _gdbm_open_file(database *, const char *);
static int _gdbm_load(database *);
static struct record_chain *_gdbm_get_records(database *, const char *);
static int _gdbm_put_record(database *, record *);
/* static */ void hello_module(void);

int _gdbm_slap(struct record *, void *);

int
database_module_init(database *d)
{
    int retval = 0;
    d->type = _this_type();
    d->open_file = _gdbm_open_file;
    d->load = _gdbm_load;
    d->get_record = _gdbm_get_records;
    d->put_record = _gdbm_put_record;

    return (retval);    
}

enum database_type
_this_type()
{
    return (GDBM); 
}

int
_gdbm_open_file(database *d, const char *path)
{
    int retval;
    FILE *ff;    
    
    printf("About to open %s\n", path);
    assert(d->database_handle == NULL);
    d->database_handle = (struct gdbm_handle) xcalloc(1, sizeof(struct ht_handle));
    CAST(d->database_handle)->table = gdbm_open("data.gdbm", 0, GDBM_WCREAT,
                                                S_IRUSR | S_IWUSR, NULL);
    ff = fopen(path, "r");
    if (ff == NULL) {
        perror("fopen failed");
        abort();
    } else {
        retval = 0;
    }
    CAST(d->database_handle)->f = ff;

    return retval;    
}

int
_gdbm_load(database *d)
{
    int retval;
    int ret;    
    
    ret = load_auth_data_f(CAST(d->database_handle)->f, _gdbm_slap,
                           CAST(d->database_handle)->table);
    retval = ret;

    return retval;
}




struct record_chain * 
_gdbm_get_records(database *d, const char *name)
{
    struct record_chain *retval;
    struct record_chain *ch;

    datum key, data;

    key.dptr = name;
    key.dsize = strlen(name);
    
    if (gdbm_exists(CAST(d->database_handle)->table, key)) {
        ch = record_chain_init(name);

        data = gdbm_fetch(CAST(d->database_handle)->table, key);
        ch->records = data.dptr;
        ch->records_len = data.dsize / sizeof(struct record);

        retval = ch;
    } else {
        retval = NULL;
    }
    return retval;
}

int
_gdbm_put_record(database *d, record *rec)
{    printf("unimplemented function\n");
    return 0;
}


void
hello_module()
{
    printf("Hello, I am the libdatabase-gdbm.so module\n");
}



int
_gdbm_slap(struct record *rec, void *_abc)
{
    struct record *tmp;
    struct record_chain *ch;
    GDBM_FILE *abc = (GDBM_FILE *) _abc;
    datum key;
    datum data, data2;

    key.dptr = rec->name;
    key.dsize = strlen(rec->name);


    RECORD_CHAIN_INIT_TMP(ch, rec->name);
    
    if (gdbm_exists(*abc, key) != 0) {
        printf("It's already there\n");
        data = gdbm_fetch(*abc, key);
        printf("We got a record chain of size %d\n", data.dsize);
        ch->records = (struct record *) data.dptr;
        ch->records_len = data.dsize / sizeof(struct record);
        record_chain_add(ch, rec);
        data2.dptr = (char *) ch->records;
        data2.dsize = ch->records_len * sizeof(struct record);
        ret = gdbm_store(*abc, key, data2, GDBM_REPLACE);
    } else {
        printf("it's not there, so let's add it\n");
        record_chain_add(ch, rec);
        data.dptr = (char *) ch->records;
        data.dsize = ch->records_len * sizeof(struct record);
        ret = gdbm_store(*abc, key, data, GDBM_INSERT);
    }
    return 0;
}



#endif
