/* Copyright (c) 2002 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 * 
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 * 
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/* $Header: /cvsroot/moodns/moodns/src/database-ht.c,v 1.6 2002/02/21 05:09:45 mawolf Exp $  */

/* When we're using a ht as the database, we don't longer have funky
   marshalling requirements like we do with db4 and gdbm.  The table instead
   store pointers directly to `struct record_chain's.  */


#ifdef HAVE_CONFIG_H
# include "../config.h"
#endif

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

#include "ht.h"
#include "record.h"
#include "auth_data.h"

#include "database.h"

#include "xmalloc.h"
#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

#define CAST(x) ((struct ht_handle *) x)
 
struct database_ht {

};

struct ht_handle {
    FILE *f;
    ht *table;
    
};

static enum database_type _this_type(void);
static int _ht_open_file(database *, const char *);
static int _ht_load(database *);
static struct record_chain *_ht_get_records(database *, const char *);
static int _ht_put_record(database *, record *);
/* static */ void hello_module(void);

int _ht_slap(struct record *, void *);

int _strcmp_local(void *, void *, void *);
void _free_local(void *, void *);
void strprint(void *);

int
database_module_init(database *d)
{
    int retval = 0;
    d->type = _this_type();
    d->open_file = _ht_open_file;
    d->load = _ht_load;
    d->get_records = _ht_get_records;
    d->put_record = _ht_put_record;
    return retval;
}


enum database_type
_this_type()
{
    return (HT);
}

int
_ht_open_file(database *d, const char *path)
{
    int retval;
    FILE *ff;

    printf("About to open %s\n", path);
    /* See parsing routines from auth_data.c.  Probably should break that
       down a bit.  Hmm.  */
    assert(d->database_handle == NULL);
    d->database_handle = (struct ht_handle *) xcalloc(1, sizeof(struct ht_handle));
    CAST(d->database_handle)->table = ht_init(_strcmp_local, _free_local);
    ff = fopen(path, "r"); 
    if (ff == NULL) {
        perror("fopen failed");
        abort();
    } else {
        retval = 0;
    }
    CAST(d->database_handle)->f = ff;

    return retval;
}

int 
_ht_load(database *d)
{
    int retval;
    int ret;

    ret = load_auth_data_f(CAST(d->database_handle)->f, _ht_slap, 
                           CAST(d->database_handle)->table);
    retval = ret;
    return retval;
}

struct record_chain *
_ht_get_records(database *d, const char *key)
{
    /* FIXME: Uhm, yeah.  */
    struct record_chain *retval;
    struct record_chain *ch; 
    
    // ch = ht_get(CAST(d->database_handle)->table, (char *) key); 
    ch = ht_get(CAST(d->database_handle)->table, strdupa(key)); 
    retval = ch; 

    return retval; 
}

int 
_ht_put_record(database *d, record *rec)
{
    int retval;
    struct record_chain *ch;

    ch = ht_get(CAST(d->database_handle)->table, rec->name);
    retval = record_chain_add(ch, rec);

    return retval;
}


void
hello_module()
{
    printf("Yo!  I'm the libdatabase-ht.la module\n");
}

 
/* FIXME: this function uses a bastardised combination of malloc and alloca.
   Eeet's reeeel faaaachted aaaap!  (I think the calls to alloca need to 
   go.)  */
int 
_ht_slap(struct record *rec, void *_abc)
{ 
    struct record_chain *ch;
    ht *abc = (ht *) _abc;



    /* printf("rec->name is %s\n", rec->name);  */

    ch = ht_get(abc, rec->name);
    if (ch != NULL) {
        /* printf("it's there already.\n");  */
        record_chain_add(ch, rec);
        /* Thanks to some shallow copying, there's no need to put this back in
           the table.  */
    } else {
        ch = record_chain_init(rec->name);
        /* printf("it is not found\n");  */
        /* printf("gonna add to chain\n");  */
        record_chain_add(ch, rec);
        /* printf("added to chain\n");  */
        ht_put(abc, rec->name, ch, NULL);
        /* printf("put it in the table\n");  */
    } 
    return 0;
}


int
_strcmp_local(void *_a, void *_b, void *_c __attribute__((unused)))
{
    return (strcmp((char *) _a, (char *) _b));
}

void
_free_local(void *_a, void *_b __attribute__((unused)))
{
    xfree(_a);
}

void
strprint(void *_arg)
{
    char *arg;
    arg = (char *) _arg;
    printf("%s", arg);
}

