/* Copyright (c) 2002 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 * 
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 * 
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
/* $Header: /cvsroot/moodns/moodns/src/database.h,v 1.4 2002/02/05 14:30:29 mawolf Exp $  */

#ifndef DATABASE_H
#define DATABASE_H

/* #include <ltdl.h>  */
#include "record.h"

enum database_type { HT = 0, GDBM, DB4, DB3 };

struct database;
typedef struct database database;

struct database {
    enum database_type type;
    /* lt_dlhandle dl_handle;  */
    
    /* Open the file containing the database.  */
    int (*open_file)(database *, const char * /* path  */); 
    
    /* Do what it takes so that a record can be found w/ get_record.  */
    int (*load)(database *);
    
    /* FIXME: This should probably return a record_chain instead.  */
    struct record_chain *(*get_records)(database *, const char *);
    
    /* Put a record into the database.  */
    /* FIXME: This might need to be adapted to use a record_chain instead. 
       Nope, it won't.  We grab the record chain, grow it, and put it back. 
       We only need a record to do that.  */
    int (*put_record)(database *, record *);
    
    /* Initially for debugging purposes; might be useful for other stuff too,
       eg when a database exists but the text file master is gone.  */
    void (*database_dump)(database *); 
    
     /* eg database handle, ht ptr  */
    void *database_handle;
};


struct database_generic;
typedef struct database_generic database_generic;



struct database_generic {
    enum database_type (*type_func)(void);
};

database_generic *database_generic_init(void);

database *database_init(char *);

void database_generic_set_type_function(database_generic *, enum database_type (*)(void));

int database_module_init(database *);

#endif
