/* Copyright (c) 2001 Omar Kilani, Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/handle_client.c,v 1.7 2002/06/19 13:33:36 mawolf Exp $  */


#ifndef _REENTRANT
#define _REENTRANT
#endif

#if HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <pwd.h>
#include <assert.h>

#include "misc.h"
#include "socket.h"
#include "database.h"
#include "cache.h"
#include "lookup.h"
#include "protocol.h"
#include "wpthread.h"
#include "byte_array.h"
#include "handle_client.h"


#include "xmalloc.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif


static long count = 0;
static pthread_mutex_t count_mutex = PTHREAD_MUTEX_INITIALIZER;

static byte_array *get_data(socket_t *);

/* Return values:
   0: Success
   1: Unspecified failure
   FIXME: This should be more specific.  */
int
handle_client(socket_t *s)
{
    int retval;
    byte_array *buf;
    packet *pa;
    int nb;
    byte_array *out;

    // inc_count(); /* <-- why do we need this?  */

    buf = get_data(s);
    if (buf == NULL) {
        retval = 1;
        goto done;
    }

    pa = packet_create_from_data(buf);
    assert(pa->size_in_bytes == buf->len);
    /* packet_debug_dump(pa);  */
    
    if (pa->h->rd) {
        /* Find out via the cache.  */
        indirect_lookup(pa); // <-- Complete this stuff!  :)
    } else {
        /* Direct lookup.  */
        direct_lookup(pa);
    }
    
    
    // printf("\nAfter we've messed with the packet structure a"
    // " bit it is:\n\n");
    // packet_debug_dump(pa);
    pa->h->qr = 1;
    pa->h->aa = 1;
    
    out = packet_convert_to_data(pa);
    
    assert(out->len == pa->size_in_bytes);
    
    /* FIXME: we need to check the return value.  */
    printf("We're going to send %d bytes (hopefully)\n", out->len);
    nb = socket_send(s, out); //->data, out->len);
    printf("We sent %d bytes\n", nb);
    {
        packet *pa2;
        pa2 = packet_create_from_data(out);
        assert(pa2->size_in_bytes == out->len);
    }
    packet_free(pa);
    
    xfree(buf); 
    retval = 0;
done:
    if (s->type == SOCKET_STREAM)
        close(s->fd);
    
    return retval; 
}


byte_array *
get_data(socket_t *s)
{
    byte_array *retval;
    byte_array *tmp1;
    int len;

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    tmp1 = (byte_array *) xcalloc(1, sizeof(byte_array));

    if (s->type == SOCKET_STREAM)  {
        // buf1 = (u8 *) alloca(2);
        if ((len = socket_recv(s, tmp1, 2)) != 2) {
            xfree(retval);
            retval = NULL;
            goto done;
        } else {
            u16 la;
            la = ntohs(*(u16 *) &(tmp1->data[0]));
            printf("la = %d\n", la);
            // buf2 = (u8 *) xcalloc(la, sizeof(u8));
            if ((len = socket_recv(s, retval/* buf2 */, la)) != la) {
                xfree(retval);
                retval = NULL;
                goto done;
            }
            // retval->data = buf2;
            // retval->len = len;
        }
    } else if (s->type == SOCKET_DATAGRAM) {
        // buf2 = (char *) xmalloc(513);
        if ((len = socket_recv(s, retval, 512)) == -1) {
            xfree(retval);
            retval = NULL;
            goto done;
        }
        // retval->data = buf2;
        // retval->len = len;
    } else {
        abort();
    }

done:
    return retval;
}



/* I still can't remember what this was meant to be used for.  

I'm sure it was a good idea at the time!  */
void
inc_count(void)
{
    wpthread_mutex_lock(&count_mutex);
    count++;
    wpthread_mutex_unlock(&count_mutex);
}

long
get_count()
{
    long retval;
    wpthread_mutex_lock(&count_mutex);
    retval = count;
    wpthread_mutex_unlock(&count_mutex);
    return retval;
}

/* "If I could only get ahead, I'd go to bed!"

-Green Apple Quickstep  */
