/*
 * Simple, robust heap / priority queue implementation.  
 *
 * Copyright (c) 2001 Michael Wolf (mike@dodgysoftware.net)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Header: /cvsroot/moodns/moodns/src/heap.h,v 1.1.1.1 2001/12/21 02:43:00 mawolf Exp $  */

#ifndef HEAP_H
#define HEAP_H

struct heap;
typedef struct heap heap;

heap *heap_init(int (*)(void *, void *, void *));

/* heap, data item, aux  */
int heap_put(heap *, void *, void *);

/* heap, aux  */
void *heap_get(heap *, void *);

/* heap, aux  */
void *heap_peek(heap *);

/* heap, function taking a piece of data and aux, aux  */
void heap_iterate(heap *, void (*)(void *, void *), void *);

/* heap, function taking a piece of data and aux, aux  */
void heap_destroy(heap *, void (*)(void *, void *), void *);

/* heap, function taking the data inputted and the aux, aux  */
void heap_set_put_hook(heap *, void (*)(void *, void *), void *);

/* heap, function taking the data returned and the aux, aux  */
void heap_set_get_hook(heap *, void (*)(void *, void *), void *);

/* heap, function taking the data returned and the aux, aux  */
void heap_set_peek_hook(heap *, void (*)(void *, void *), void *);

/* print out what we've got  */
void heap_debug_dump(heap *, void(*)(void *));

#endif

