/* this file is slated for removal from moodns, to be replaced by moolog.  */

/* Copyright (c) 2001 Michael Wolf.  All rights reserved. 

This file is part of moodns.

moodns is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version. 

moodns is distributed in the hope that it will be useful,  
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with moodns; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


/* $Header: /cvsroot/moodns/moodns/src/log.c,v 1.6 2002/07/20 16:55:43 mawolf Exp $  */


#ifndef _REENTRANT
#define _REENTRANT
#endif

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h> /* freebsd specific?  */
#include <time.h>
#include <netdb.h> /* Solaris stores MAXHOSTNAMELEN here  */
#include <unistd.h>



#include "log.h"

#include "xmalloc.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif

struct log {
    FILE *handle;
    char *hostname;
    char *progname;
    char ctime_r_buf[27];
};

void chomp(char *);

log *
log_init(const char *path, char *name)
{
    log *retval;
    FILE *f;
        
    f = fopen(path, "a");
    if (f == NULL) 
        retval = NULL;
    else 
        retval = log_init_file(f, name);
        
    return (retval);
}

log *
log_init_file(FILE *f, char *name)
{
    log *retval;
    time_t now;    
    
    retval = (log *) xcalloc(1, sizeof(log));
    retval->handle = f;
    
    if (name == NULL) 
        retval->progname = xstrdup("");
    else
        retval->progname = name;
    
    retval->hostname = (char *) xcalloc(MAXHOSTNAMELEN, sizeof(char));
    
    if (gethostname(retval->hostname, MAXHOSTNAMELEN) < 0) {
        perror("gethostname() failed");
        abort();
    }
    
    now = time(NULL);
    ctime_r(&now, retval->ctime_r_buf);
    chomp(retval->ctime_r_buf);
    fprintf(retval->handle, "%s %s: log opened\n", 
            retval->ctime_r_buf, retval->progname);
    
    return (retval);
}

log *
log_init_fd(int fd, char *name)
{
    log *retval;
    FILE *f;
    
    f = fdopen(fd, "a");
    if (f == NULL)
        retval = NULL;
    else 
        retval = log_init_file(f, name);
    
    return (retval);
}   

int
log_printf(log *l, const char *fmt, ...)
{
    va_list ap;
    time_t now;
    now = time(NULL);
    ctime_r(&now, l->ctime_r_buf);
    chomp(l->ctime_r_buf);
    fprintf(l->handle, "%s %s: ", l->ctime_r_buf, l->progname);
    va_start(ap, fmt);
    return vfprintf(l->handle, fmt, ap);
    va_end(ap);

    fflush(l->handle);
}

int
log_close(log *l)
{
    fflush(l->handle);
    fclose(l->handle);
    /* maybe should deallocate memory too  */
    return 0;
}

void
chomp(char *c)
{
    char *x;
    x = strchr(c, '\n');
    if (x != NULL)
        *x = '\0';

}

#ifdef TEST

int
main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
{
    log *l;

    l = log_init("test.logfile", "log test");
    
    log_printf(l, "this is the number 5: %d\n", 5);
    log_printf(l, "%s %s\n", "foo", "bar");
    log_printf(l, "let's dump a log\n");

    

    return 0;
}

#endif
