/* Copyright (c) 2001 Omar Kilani, Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/main.c,v 1.17 2002/02/21 05:16:21 mawolf Exp $  */

#ifndef _REENTRANT
#define _REENTRANT
#endif

#if HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <pwd.h>

#include "wpthread.h"
#include "pool.h"

#include "misc.h"

#include "socket.h"
#include "options.h"
#include "log.h"
#include "cache.h"
#include "record.h"
#include "security.h"
#include "database.h"
#include "handle_client.h"
#include "protocol.h"

#include "xmalloc.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif

#ifdef __FreeBSD
char *malloc_options = "AJR";
#endif

struct connection_wrappers {
    socket_t *tcp;
    socket_t *udp;

    pthread_mutex_t mutex;
    pthread_cond_t cv;
};

// static void signal_handler(int);
// static void initialise_roots(void);
static void *handle_connections(void *);
/* static void *handle_client(void *v);  */



cache *c;
database *d;
pool *p;
log *l;

record_chain *roots;

const char *MODULE_PATH = "libdatabase-ht.la";

struct global_options globals;

int
main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
{
    struct connection_wrappers sh;
    int i;
    socket_address_t sat;    

    srandom(time(NULL));

    printf("%d\n", getpid());
    /* wpthread_mutex_lock(&ltdl_mutex);
       lt_dlinit();
       wpthread_mutex_unlock(&ltdl_mutex);  */
    options_init();

    memset(&sh, 0, sizeof(struct connection_wrappers));

    sat.internet.port = globals.net.port;
    sat.internet.address = NULL;
    sh.tcp = socket_listener(&sat, SOCKET_INTERNET, SOCKET_STREAM);
    sh.udp = socket_listener(&sat, SOCKET_INTERNET, SOCKET_DATAGRAM);

    /* This works w/ ht only.  Maybe we should make it work with more 
       database types.  However, don't bother with it till everything else
       that's important works.  */
    d = database_init(NULL /*MODULE_PATH*/);
    d->open_file(d, "../doc/data");
    d->load(d);

    drop_privs();
    
    wpthread_mutex_init(&sh.mutex, NULL);
    wpthread_cond_init(&sh.cv, NULL);
    /* signal(SIGSEGV, signal_handler);  */
    
    if (globals.cache.use_cache == 1)
        c = cache_init();

    p = pool_init(handle_connections, &sh);
    for (i = 0; i < globals.general.num_threads; ++i) {
        pool_increment(p);
    }

    while (1) {
        sleep(10);
        /* Log stats periodically.  */
    }
}

void *
handle_connections(void *_arg)
{
    struct connection_wrappers *sh;
    fd_set fullset;

    sh = (struct connection_wrappers *) _arg;

    if (!sh->tcp || !sh->udp) {
        printf("we're not talking to the network!\n");
        abort();
    }

    FD_ZERO(&fullset);
    FD_SET(sh->tcp->fd, &fullset);
    FD_SET(sh->udp->fd, &fullset);
    
    while (1) {
        int s = 0;
        fd_set ready;
        struct timeval timeout;

        wpthread_mutex_lock(&sh->mutex);
        ready = fullset;
        timeout.tv_sec = 60;
        timeout.tv_usec = 0;

        if ((s = select(FD_SETSIZE, &ready, 0, 0, &timeout)) < 0) {
            perror("select failed");
            wpthread_mutex_unlock(&sh->mutex);
            continue;
        }

        if (s == 0) {
            fprintf(stderr, "DEBUG: no data waiting.\n");
            wpthread_mutex_unlock(&sh->mutex);
            continue;
        }

        if (FD_ISSET(sh->tcp->fd, &ready)) {
            socket_t *client = socket_accept(sh->tcp);
            handle_client(client);
        }

        if (FD_ISSET(sh->udp->fd, &ready)) {
            socket_t *client = socket_accept(sh->udp);
            handle_client(client);
        }

        wpthread_mutex_unlock(&sh->mutex);
    }
}


#if 0

void
signal_handler(int sig)
{
    pid_t pid;
    switch (sig) {
    case SIGSEGV:
        /* This trick apparently works on Linux.  It doesn't seem to work on
           FreeBSD, but FreeBSD does seem to give you a core dump that's
           in the "right place".  

           The intention is that there'll only be one core dump, and it'll
           be from the forked child and hence from the offending thread.  */
        printf("hi\n");
        pid = fork();
        printf("hmm\n");
        if (pid == 0) {
            printf("I should die.\n");
            abort();
        } else {
            wait(NULL);
            printf("about to go\n");
            exit(1); 
        }
        break;
    default:
    }
}

#endif
