/*
 * The merrimack process id file library.
 *
 * Copyright (c) 2001 Michael Wolf (mike@dodgysoftware.net)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * This file is part of the ironclad suite of tools to create and manipulate
 * long-running daemon style programs on Unix.
 *
 * For a more thorough treatment, see 
 *     http://dodgysoftware.net/ironclad/merrimack.html
 */

/* $Header: /cvsroot/moodns/moodns/src/merrimack.h,v 1.1 2002/06/19 13:36:43 mawolf Exp $  */

#ifndef MERRIMACK_H
#define MERRIMACK_H

#include <sys/types.h>

struct mpid;
typedef struct mpid mpid;

typedef enum mpid_create_errors {
    NONE = 0,
    ALREADY,
    PERMISSION,
    CONSISTENCY,
    RESOURCES,
    WEIRDNESS
} mpid_create_errors;

/* Create a mpid object.  The first argument is the name of the program as it
 * is expected to be executed, and the second is the path to where a pidfile
 * should be created (/var/run/foo.pid?).  
 *
 * In the event that another instance of the same program is already running,
 * return NULL.  
 */
int mpid_create(char *, pid_t, int *);

/*
 * Clean up any resources (memory, locks, etc) associated with an mpid. 
 */
int mpid_destroy(char *, pid_t, int *);

/*
 * Get the status of a program which may have a given pid file.
 */
int mpid_status(char *, char *, int *);

#endif











