/*
 * Copyright (c) 2001 Michael Wolf, Omar Kilani  All rights reserved. 
 * 
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.     
 * 
 * moodns is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
  
    
#ifndef MISC_H
#define MISC_H

#include <stdio.h>

#ifdef __cplusplus
# define BEGIN_C_DECLS extern "C" {
# define END_C_DECLS }
#else
# define BEGIN_C_DECLS
# define END_C_DECLS
#endif

BEGIN_C_DECLS;

/* I think this crap is system dependent.  As we port to different systems,
   we'll want to gneralise these typedefs.  */
typedef enum { FALSE = 0, TRUE } bool;
typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef unsigned long long u64;
typedef bool dookie;

char *afgetln(FILE *);

/* The code uses the ``unused'' attribute a lot.  Using -Wno-unused is a bad
   idea, but sometimes you'll get variables which aren't used but about which
   you can't do anything.  In dss, for example, this is used for argc and argv 
   in main() and in the local_strcmp and local_free when we use ht.  */
#ifdef __GNUC__
# define __ATTRIBUTE__(X) __attribute__((X))
# define __INLINE__ inline
/* # define __FILE_LINE_FUNC__ __FILE__ ":" __LINE__ ":" __FUNCTION__  */
#else  
# define __ATTRIBUTE__(X)
# define __INLINE__ 
/* # define __FILE_LINE_FUNC__ __FILE__ ":" __LINE__  */
#ifndef __FUNC__
# define __FUNC__ "unknown function"
#endif
#endif  

#ifndef __GNUC__
#define __FUNCTION__ "[unknown function - consider compiling with gcc]"
#endif


#define UNIMPLEMENTED \
    printf("Warning, unimplemented feature: \n File: %s\n " \
           "Line: %d\n Function: %s\n", \
           __FILE__, __LINE__, __FUNCTION__);

void initialise_roots(void);
void inc_count(void);
long get_count(void);

void bitprint_u8(u8);
void bitprint_u16(u16);
void bitprint_u32(u32);

u32 bogo_htonl(u32);
u32 bogo_ntohl(u32);
u16 bogo_htons(u16);
u16 bogo_ntohs(u16);
void pname(u32);

/* Neither of these is ANSI C compliant.  Can it be done?  

   I think that strdupa2 is part of glibc as strdupa, while strdupa is my 
   invention.  But I could be mistaken.  */
#define strdupa(n) (char *) ({ \
    char *__x__; \
    __x__ = alloca(strlen(n) + 1); \
    strcpy(__x__, n); \
})

#define strdupa2(s) (__extension__ \
                     ({__const char *__old = (s); \
                       size_t __len = strlen(__old) + 1; \
                       char *__new = alloca(__len); \
                       (char *) memcpy(__new, __old, __len); \
}))

#define xalloca(p, s) \
    p = alloca(s); \
    if (p == NULL) abort(); \
    memset(p, 0, sizeof(*p));


END_C_DECLS;

#endif
