/* Copyright (c) 2002, Michael Wolf.  All rights reserved.  */
/* Copyright (c) 2002, Ximian Inc.  All rights reserved.  */

/* Usage: moodaemon -d DETACH/NODETACH -- file [args ...]

run a program, either as a daemon, or in the foreground.  

The following signals are caught and immediately sent to the child process:
SIGHUP, SIGUSR1, SIGUSR2

The following signals are caught and immediately sent to the child process.  
After that, moodaemon waits for the child process to exit.  (FIXME: make the 
amount of time to wait tunable, after which moodaemon kills the process.)
SIGTERM, SIGINT, SIGQUIT, SIGKILL.


 
FIXME: 
should fork twice, so that there is always a moodaemon running

should do pidfiles

we're not doing waiting correctly.  in fact, what we are doing is waaaaaay off
base.

improve recognition of when the child process fails to run.  (see primed for
code that does this correctly.)

convert command line arguments to getopt.

*/

/* $Header: /cvsroot/moodns/moodns/src/moodaemon.c,v 1.1 2002/06/19 13:51:25 mawolf Exp $  */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <errno.h>
#include <assert.h>

#include "spawn.h"
#include "safewait.h"
#include "util.h"

#include "xmalloc.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif

pid_t pid, pid2;
struct rusage ru;

int do_detach = -1;
char *program = NULL;
char **program_args = NULL;

void set_signal_handlers(void);
void signal_handler(int);
void process_arguments(int, char **);
void run_target(void);

int
main(int argc, char **argv)
{
    int ret;
    int status;
    int c;

    while ((c = getopt(argc, argv, "d:")) != EOF) {
        switch (c) {
        case 'd':
            printf("boo!\n");
            if (strcmp("detach", optarg) == 0) 
                do_detach = 1;
            else if (strcmp("nodetach", optarg) == 0) 
                do_detach = 0;
            else 
                die("invalid argument to -d\n");
            break;
        default:
            die("unrecognised option\n");
        }
    }
    
    if (do_detach == -1)
        die("don't know whether to detach or not\n");

    printf("optind = %d\n", optind);
    process_arguments(argc - optind, &argv[optind]);

    memset(&ru, 0, sizeof(ru));
    set_signal_handlers();

    if (do_detach == -1)
        croak("do_detach improperly set\n");

    pid = fork();
    if (pid == -1) {
        croak("unable to fork child process");
    } else if (pid == 0) {
        pid2 = fork();
        switch (pid2) {
        case -1:
            croak("unable to fork child process");
            break;
        case 0:
            run_target();
            break;
        default:
            set_signal_handlers();
            ret = safewait(pid2, &status, 0, &ru, 0, -1);
            /* maybe print something based on the contents of ru  */
            exit(status);
            break;
        }
    } else {
        if (do_detach == 1) {
            exit(0);
        } else {
            waitpid(pid, &status, 0);
            exit(0);
        }
    }

    return 0;    
}

void
process_arguments(int argc, char **argv)
{
    int i;

    program = argv[0];
    
    printf("program is %s\n", program);

    if (argc > 1) {
        program_args = (char **) xcalloc((argc - 1) + 1, sizeof(char *));
        for (i = 0; i < (argc - 1) + 1; ++i) {
            program_args[i] = xstrdup(argv[i]);
        }
        program_args[i] = NULL;
    } else {
        program_args = (char **) xcalloc(1, sizeof(char *));
        program_args[0] = NULL;
    }
}

void
run_target()
{
    int ret;
    pid_t ppid;
    
    if (do_detach == 1) {
        // printf("hoongah\n");
        fclose(stdout);
        fclose(stderr);
        fclose(stdin);
        setsid();
    } else {
        // printf("doongah\n");
    }
    printf("alksjfrlsd\n");
    ret = execvp(program, program_args);
    // system("echo fork | wall");
    /* not good  */
    ppid = getppid();
    kill(ppid, SIGUSR1);
    croak("failed to run program: %s", strerror(errno));
}

void
set_signal_handlers() 
{
    signal(SIGHUP, signal_handler);
    signal(SIGUSR1, signal_handler);
    signal(SIGUSR2, signal_handler);
    
    signal(SIGTERM, signal_handler);
    signal(SIGINT, signal_handler);
    signal(SIGQUIT, signal_handler);
    signal(SIGKILL, signal_handler);
}


void
signal_handler(int sig)
{
    int ret;
    int status;
    /* puts("hello");  */

    switch (sig) {
    case SIGHUP:
    case SIGUSR1:
    case SIGUSR2:
        kill(pid2, sig);
        break;
    case SIGTERM:
    case SIGINT:
    case SIGQUIT:
    case SIGKILL:
        kill(pid2, sig);
        ret = wait4(pid, &status, 0, &ru);
        exit(status);
        break;
    default:
        croak("unknown signal");
        break;
    }
}






