/* Copyright (c) 2001 Michael Wolf  */
/* Copyright (c) 2002 Ximian, Inc.  */
/* Copyright (c) 2002 Michael Wolf  */

/* By Michael Wolf <maw@ximian.com> */

/* Usage: moolock -p PID -f LOCKFILE -a LOCK-OR-UNLOCK  */

/* Background: many long running programs, such as servers, only should be 
   running in one instance.  This is generally because they rely on exclusive
   access to a resource, such as a TCP port.  A common technique to realise
   this is to create a ``pid-file'' (sometimes storing the process ID of the
   currently running program).  When a new instance of that program is started,
   it will look for that pid-file and assert that the process id contained 
   there is not running.
   
   While many programs do indeed do this, most of them do not do it properly.
   This program, moolock, is an attempt to make proper pid-files easy to 
   use and manipulate.  At the same time, it attempts to be portable; the same
   interface to moolock is intended to work on any Unix or Unix-like system.

   Like most locking mechanisms, use of moolock is strictly advisory.

   moolock is based on code originally written for the ironclad suite, 
   specifically the merrimack module.  However, moolock is a seperate 
   executable, not a library.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>
#include <syslog.h>

#include "merrimack.h"
#include "util.h"

static pid_t pid;
static char *path;
static int lock;
static int unlock;

static void create(void);
static void destroy(void);
static void usage(void);

int
main(int argc, char **argv)
{
    int c;
        
    pid = 0;
    lock = -1;
    unlock = -1;

    while ((c = getopt(argc, argv, "p:f:a:h")) != EOF) {
        switch (c) {
        case 'p':
            pid = atoi(optarg);
            break;
        case 'f':
            path = optarg;
            break;
        case 'a':
            if (strcmp(optarg, "lock") == 0) {
                lock = 1;
            } else if (strcmp(optarg, "unlock") == 0) {
                unlock = 1;
            } else {
                die("tell me to lock or unlock");
            }
            break;
        case 'h':
            usage();
            exit(0);
            break;
        default:
            die("hmm");
            break;
        }
    }
    
    if (lock == 1 && unlock == 1) {
        die("i can only lock or unlock; i can't do both\n");
    }
    if (lock == -1 && unlock == -1) {
        die("tell to either lock or unlock\n");
    }
    
    if (lock == 1) 
        create();
    if (unlock == 1)
        destroy();
    
    /* We should never actually make it here.  */
    abort();
    return 1;

}

void
create()
{
    int ret;
    int errors;

    ret = mpid_create(path, pid, &errors);
    if (ret == 0) {
        printf_on_debug("mpid_create failed with error code %d\n", errors);
        exit(EXIT_FAILURE);
    }
}

void
destroy()
{
    int ret;
    int errors;

    ret = mpid_destroy(path, pid, &errors);
    if (ret == 0) 
        exit(EXIT_FAILURE);
    else
        exit(EXIT_SUCCESS);
}

void
usage()
{
    printf("Usage: moolock -p PID -f LOCKFILE -a LOCK-OR-UNLOCK\n");
}
