/* Copyright (c) 2002, Michael Wolf.  All rights reserved.  */
/* Copyright (c) 2002, Ximian Inc.  All rights reserved.  */

/* Usage: moolog [-debug] file1 file2 program [args ...]

Run program, redirecting its standard out to file1 and its standard error 
to file2.  file1 and file2 can be the same file.  file1 or file2 or both may
be /dev/null.  Does not daemonize.  Use moodaemon for that purpose. 

FIXME: Some signals should be sent to the child process.  Which ones?

*/

/* $Header: /cvsroot/moodns/moodns/src/moolog.c,v 1.1 2002/06/19 14:06:41 mawolf Exp $  */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <poll.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <errno.h>
#include <assert.h>

#include "misc.h"
#include "util.h"

#include "xmalloc.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif

#define MAX(a, b) (a) > (b) ? (a) : (b)

int debug;

char *file1, *file2;
char *program = NULL;
char **program_args = NULL;
FILE *out, *err;

void signal_handler(int);
void process_arguments(int, char **);
void run_target(void);

int
main(int argc, char **argv)
{
     pid_t pid;
    // int status;
    int p1[2], p2[2];
    // struct rusage ru;
    FILE *foo, *bar;
    // fd_set fdset;
    struct pollfd pfd[2];
    int ret;
    int max;
    char *d;
    int od, ed;

    signal(SIGCHLD, signal_handler);
    // memset(&ru, 0, sizeof(ru));
    
    process_arguments(argc, argv);

    out = fopen(file1, "a");
    err = fopen(file2, "a");
    
    pipe(p1);
    pipe(p2);
    
    switch (pid = fork()) {
    case -1:
        croak("fork() failed: %s\n", strerror(errno));
        break;
    case 0:
        ret = dup2(p1[1], 1);
        ret = dup2(p2[1], 2);
        // close(p1[0]);
        // close(p2[0]);
        run_target();
        break;
    default:
        // close(p1[1]);
        // close(p2[1]);
        foo = fdopen(p1[0], "r");
        bar = fdopen(p2[0], "r");
        max = MAX(p1[0], p2[0]);

        od = ed = 0;

        while (!od && !ed) {
            memset(&pfd[0], 0, sizeof(pfd[0]));
            memset(&pfd[1], 0, sizeof(pfd[1]));
            pfd[0].fd = p1[0];
            pfd[0].events = POLLIN;
            pfd[1].fd = p2[0];
            pfd[1].events = POLLIN;

            printf("going to poll\n");
            ret = poll(pfd, 2, 60000);
            printf("poll returned %d\n", ret);
            switch (ret) {
            case -1:
                /* hmm */
                break;
            case 0:
                printf("timeout\n");
                break;
            default:
                if (pfd[0].revents & POLLIN) {
                    d = afgetln(foo);
                    if (d) {
                        printf("%s\n", d);
                        fprintf(out, "%s\n", d);
                        fflush(out);
                        xfree(d);                
                    }
                    if (feof(foo))
                        od = 1;
                }
                if (pfd[1].revents & POLLIN) {
                    d = afgetln(bar);
                    if (d) {
                        printf("%s\n", d);
                        fprintf(err, "%s\n", d);
                        fflush(err);
                        xfree(d);                
                    }
                    if (feof(bar))
                        ed = 1;
                }
                break;
            }
            printf("end of loop\n");
        }
        break;
    }




    return 0;    
}

void
process_arguments(int argc, char **argv)
{
    int i; //, x;

    if (argc < 5) 
        die("too few arguments\n");
    
    if (!strcmp(argv[1], "debug")) {
        debug = 1;
    } else if (!strcmp(argv[1], "nodebug")) {
        debug = 0;
    } else {
        die("do we debug or not?\n");
    }

    file1 = argv[2];
    file2 = argv[3];
    
    program = argv[4];
    if (argc > 5) {
        program_args = (char **) xcalloc((argc - 5) + 1, sizeof(char *));
        for (i = 0; i < (argc - 5); ++i) {
            // printf("copying %s\n", argv[i + 5]);
            program_args[i] = xstrdup(argv[i + 5]);
        }
        program_args[i] = NULL;
    } else {
        program_args = (char **) xcalloc(1, sizeof(char *));
        program_args[0] = NULL;
    }

    // printf("file1 is %s\n", file1);
    // printf("file2 is %s\n", file2);
    // printf("program is %s\n", program);
    // for (x = 0; x < i; ++x) {
    //     printf(" %s", program_args[x]);
    // }
    // printf("\n");
}

void
run_target()
{
    int ret;
    pid_t ppid;
    
    ret = execvp(program, program_args);
    /* not good  */
    ppid = getppid();
    kill(ppid, SIGUSR1);
    system("echo fuck | wall");
    croak("failed to run program");
}

void
signal_handler(int sig)
{
    puts("hello");
    switch (sig) {
    case SIGUSR1:
        croak("failed to run child process");
        break;
    case SIGUSR2:
        break;
    case SIGCHLD:
        croak("child process crashed");
        break;
    case SIGALRM:
        /* looks ok.  */
        exit(0);
    default:
        croak("unhandled signal handled (?)");
        break;
    }
}






