/* Copyright (c) 2001 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/mooquery.c,v 1.10 2002/03/05 02:08:24 mawolf Exp $  */

/* Usage: mooquery -s server -n name -t type -p port -m tiMeout [-r] */


#if HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <pwd.h>
#include <assert.h>

#include "wpthread.h"
#include "pool.h"

#include "misc.h"

#include "socket.h"
#include "options.h"
#include "log.h"
#include "cache.h"
#include "record.h"
#include "security.h"
#include "database.h"
#include "query.h"
#include "protocol.h"

#include "xmalloc.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif

char *malloc_options = (char *) &"AJR";

char *server = (char *) &"127.0.0.1";

char *name = NULL;
char *name_real;

char *type = (char *) &"a";
int type_real;

char *port = (char *) &"53";
u16 port_real;

char *timeout = (char *) &"60";
u32 timeout_real;

int recursion_desired = 0;

void prout_usage(void);
int do_stuff(void);
void to_real(void);
int recordtype(char *);

int
main(int argc, char **argv)
{
    int c;
    int oy;

    oy = 0;
    while ((c = getopt(argc, argv, "s:n:t:p:m:u:r")) != -1) {
        oy = 1;
        switch (c) {
        case 's':
            server = optarg;
            break;
        case 'n':
            name = optarg;
            break;
        case 't':
            type = optarg;
            break;
        case 'p':
            port = optarg;
            break;
        case 'm':
            timeout = optarg;
            break;
        case 'r':
            recursion_desired = 1;
            break;
        case 'u':
            prout_usage();
            break;
        default:
            printf("unrecognised option\n");
            prout_usage();
            break;
        }
    }

    if (oy == 0) {
        prout_usage();
    }

    if (name == NULL) {
        printf("you need to provided a name to look up, if nothing else\n");
        return 2;
    }

    return do_stuff();
}

void 
prout_usage()
{
    printf("usage: ");
    puts("mooquery -s server -n name -t type -p port -m tiMeout [-r]");
    puts("(as a bare minimum, you might get by just using a -n option)");
    puts("(-r specifies that you want a recursive query)");
    exit(1);
}

int
do_stuff()
{
    packet *p;
    struct header *h;
    struct question *q;
    packet *r;
    
    to_real();

    p = (packet *) xcalloc(1, sizeof(packet));
   
    h = (struct header *) xcalloc(1, sizeof(struct header));
    h->id = 3;
    h->qr = 0;
    h->opcode = 0;
    h->rd = recursion_desired;
    h->qdcount = 1;

    p->size_in_bytes += header_size(h);
    
    q = (struct question *) xcalloc(1, sizeof(struct question));
    q->qname = xstrdup(name_real);
    q->qtype = type_real;
    q->qclass = IN;

    p->h = h;
    p->qd = (struct question **) xrealloc(p->qd, 1 * sizeof(struct question *));
    p->qd[0] = q;
    
    p->size_in_bytes += question_size(q);

    r = query_send_char(p, server, port_real, timeout_real);
    
    if (r)
        packet_debug_dump(r);

    return 0;
}

void
to_real()
{
    name_real = name;

    type_real = recordtype(type);

    timeout_real = strtoul(timeout, NULL, 10);

    port_real = atoi(port);
}

int 
recordtype(char *in)
{
    int retval;
    
    if (!strcasecmp("A", in)) {
        retval = A;
    } else if (!strcasecmp("NS", in)) {
        retval = NS;
    } else if (!strcasecmp("CNAME", in)) {
        retval = CNAME;
    } else if (!strcasecmp("SOA", in)) {
        retval = SOA;
    } else if (!strcasecmp("PTR", in)) {
        retval = PTR;
    } else if (!strcasecmp("MX", in)) {
        retval = MX;
    } else if (!strcasecmp("TXT", in)) {
        retval = TXT;
    } else {
        puts("currently unsupported option");
        exit(4);
    }

    return retval;
}
