/* 
 * Part of the pthread_utils package.  
 *
 * Copyright (c) 2001 Michael Wolf.  All rights reserved.  
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Header: /cvsroot/moodns/moodns/src/pool.h,v 1.2 2001/12/21 13:28:34 mawolf Exp $  */

#ifndef POOL_H
#define POOL_H

struct pool;
typedef struct pool pool;

/* I'm not sure which would be better:
   - having a second void * argument for pool_init which is passed to 
   new threads
   - having a second void * argument for pool_increment which is passed to
   new threads
   - having a second void * argument in each, where the argument given to
   pool_increment overrides the argument given to pool_init, and where 
   there's a magic pointer constant which means "use the default".  

   I think that the first is the correct solution.  But I'm not convinced.  */

/* This could be used as part of the third option above.  How hacky.  */
extern int POOL_MAGICK_DEFAULT;
#define POOL_DEFAULT (void *) &POOL_MAGICK_DEFAULT;

pool *pool_init(void *(*)(void *), void *);
void pool_status(pool *, int *, int *);
int pool_increment(pool *); 
int pool_decrement(pool *);
int pool_destroy(pool *);

int pool_check(pool *);

#endif
