/* Copyright (c) 2001, 2002 Omar Kilani, Michael Wolf.  All rights reserved.
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * moodns is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/protocol-header.c,v 1.4 2002/07/20 17:02:37 mawolf Exp $  */

/* Big picture:

This file implements operations on the header of a dns packet.  */

/* Based on information presented in RFC 1035, by P Mockapetris.  */

#ifndef _REENTRANT
#define _REENTRANT
#endif

#if HAVE_CONFIG_H
#  include "../config.h"
#endif

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/param.h>  /* FreeBSD needs this for htons etc  */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "record.h"
#include "byte_array.h"
#include "ht.h"
#include "protocol.h"
#include "protocol-header.h"

#include "xmalloc.h"
#if WITH_DMALLOC
#  include <dmalloc.h>
#endif

/* Convert a header into its packed equivalent.  */

int 
header_pack(struct header *orig, struct header_packed *packed)
{
    assert(orig != NULL && packed != NULL);

    if (orig->z != 0)
        abort();

    memset(packed, 0, sizeof(*packed));

    /* Bytes 0, 1  */
    *(u16 *) & packed->data[0] = htons(orig->id);

    /* Byte 2  */
    packed->data->data[2] |= orig->qr << 7;
    packed->data->data[2] |= orig->opcode << 3;
    packed->data->data[2] |= orig->aa << 2;
    packed->data->data[2] |= orig->tc << 1;
    packed->data->data[2] |= orig->rd;

    /* Byte 3  */
    packed->data->data[3] |= orig->ra << 7;
    packed->data->data[3] |= orig->rcode;

    /* Bytes 4, 5  */
    *(u16 *) & packed->data->data[4] = htons(orig->qdcount);

    /* Bytes 6, 7  */
    *(u16 *) & packed->data->data[6] = htons(orig->ancount);

    /* Bytes 8, 9  */
    *(u16 *) & packed->data->data[8] = htons(orig->nscount);

    /* Bytes 10, 11  */
    *(u16 *) & packed->data->data[10] = htons(orig->arcount);

    return (0);
}

/* Turn a packed header into its unpacked equivalent.  */

int 
header_unpack(struct header *unpacked, struct header_packed *orig)
{
    assert(unpacked != NULL && orig != NULL);

    memset(unpacked, 0, sizeof(*unpacked));

    unpacked->id = ntohs(*(u16 *) & orig->data[0]);
    unpacked->qr = orig->data->data[2] >> 7;
    unpacked->opcode = orig->data->data[2] >> 3;
    unpacked->aa = orig->data->data[2] >> 2;
    unpacked->tc = orig->data->data[2] >> 1;
    unpacked->rd = orig->data->data[2];
    unpacked->ra = orig->data->data[3] >> 7;
    unpacked->z = orig->data->data[3] >> 4;
    unpacked->rcode = orig->data->data[3];
    unpacked->qdcount = ntohs(*(u16 *) & orig->data->data[4]);
    unpacked->ancount = ntohs(*(u16 *) & orig->data->data[6]);
    unpacked->nscount = ntohs(*(u16 *) & orig->data->data[8]);
    unpacked->arcount = ntohs(*(u16 *) & orig->data->data[10]);

    return 0;
}


size_t 
header_size(struct header *h)
{
    return sizeof(*h);
}
