/* Copyright (c) 2001, 2002 Omar Kilani, Michael Wolf.  All rights reserved.
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * moodns is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/protocol-rr.c,v 1.5 2002/07/20 17:02:37 mawolf Exp $  */

/* This file is full of tedious code dealing with the conversion of
   resource records to an from data.  A single error in this file,
   when invoked, will propagate throughout conversion.  So don't make
   errors.  

   The functions add_{answer,authoritative,additional}_from_data are
   called outside of this file.  They in turn call on
   build_rr_from_data.  That function takes a raw data argument, a
   success or failure argument, and a pointer to the base of the
   data.  The pointer to the base is necessary to perform
   decompression since offsets are used throughout.   

   record_to_data_X: Convert a record to data, suitable for appending
   to some outgoing data.  

   rr_to_record_X: Convert the rdata part of a struct resource_record
   into the record.record part of a struct record.  
   
   The functions record_to_data_* and rr_to_record_* are almost
   opposites of each other.  */

/* Known issues:

   - This file is a fucking mess!
   
   - Not all record types are handled.  

   - Won't compile with -ansi.  Definitely won't compile with -pedantic.  

 */

/* Based on information presented in RFC 1035, by P Mockapetris.  */

#ifndef _REENTRANT
#define _REENTRANT
#endif

#if HAVE_CONFIG_H
#  include "../config.h"
#endif

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/param.h>  /* FreeBSD needs this for htons etc  */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "record.h"
#include "byte_array.h"
#include "ht.h"
#include "protocol.h"
#include "protocol-rr.h"


#include "xmalloc.h"
#if WITH_DMALLOC
#  include <dmalloc.h>
#endif

static struct resource_record *build_rr_from_data(const u8 *, size_t *, const u8 *);

static byte_array *record_to_data_a(record *);
static byte_array *record_to_data_ns(record *, int *, ht *);
static byte_array *record_to_data_mx(record *, int *, ht *);
static byte_array *record_to_data_cname(record *, int *, ht *);
static byte_array *record_to_data_txt(record *);
static byte_array *record_to_data_soa(record *);
static byte_array *record_to_data_ptr(record *);
static byte_array *record_to_data_hinfo(record *);

static void rr_to_record_a(record *, byte_array *);
static void rr_to_record_ns(record *, byte_array *);
static void rr_to_record_mx(record *, byte_array *);
static void rr_to_record_cname(record *, byte_array *);
static void rr_to_record_txt(record *, byte_array *);
static void rr_to_record_soa(record *, byte_array *);
static void rr_to_record_ptr(record *, byte_array *);
static void rr_to_record_hinfo(record *, byte_array *);


/* Convert the idxth answer section of a packet to raw data.  */

byte_array * 
packet_convert_answer_to_data(packet *p, int idx, 
                              int *offset __attribute__((unused)), 
                              ht *locations __attribute__((unused)))
{
    byte_array *out;
    byte_array *tmp;
    
    out = (byte_array *) xcalloc(1, sizeof(byte_array));

    tmp = name_to_bytes(p->an[idx]->name, offset, locations);
    byte_array_cat(out, tmp);
    byte_array_catu16(out, htons(p->an[idx]->type));
    byte_array_catu16(out, htons(p->an[idx]->class));
    byte_array_catu32(out, htonl(p->an[idx]->ttl));
    byte_array_catu16(out, htons(p->an[idx]->rdlength));
    byte_array_cat(out, p->an[idx]->rdata);
    
    byte_array_destroy(tmp);

    return out;
}

byte_array *
packet_convert_authoritive_to_data(packet * p __attribute__((unused)),
                                   int idx __attribute__((unused)), 
                                   int *offset __attribute__((unused)),
                                   ht *locations __attribute__((unused)))
{
    byte_array *out;

    out = (byte_array *) xcalloc(1, sizeof(byte_array));

    UNIMPLEMENTED;

    return out;
}

byte_array * 
packet_convert_additional_to_data(packet * p __attribute__((unused)),
                                  int idx __attribute__((unused)),
                                  int *offset __attribute__((unused)),
                                  ht *locations __attribute__((unused)))
{
    byte_array *out;

    out = (byte_array *) xcalloc(1, sizeof(byte_array));

    UNIMPLEMENTED;
    return out;
}

int
packet_add_rr(packet *p __attribute__((unused)), 
              u8 *name __attribute__((unused)), 
              u32 type __attribute__((unused)), 
              u32 class __attribute__((unused)), 
              u32 tll __attribute__((unused)),
              u32 rdlength __attribute__((unused)), 
              u8 *rdata __attribute__((unused)))
{
    UNIMPLEMENTED;
    return 0;
}

struct resource_record *
build_rr_from_data(const u8 *data, size_t *size, const u8 *base)
{
    struct resource_record *retval;
    struct resource_record *rr;
    int offset;
    u16 len;
    int dsize;
    byte_array *tmp;
    
    rr = (struct resource_record *) xcalloc(1, sizeof(struct resource_record));

    offset = 0;

    /* Get the name.  */
    /* XXX: We need to deal with name compression here.  */
    rr->name = question_arr_to_char(data, base, &dsize);
    offset += dsize;

    /* Get the type.  */
    rr->type = ntohs(*(const u16 *) &data[offset]);
    offset += 2;

    /* Get the class.  */
    rr->class = ntohs(*(const u16 *) &data[offset]);
    offset += 2;

    /* Get the ttl.  */
    rr->ttl = ntohl(*(const u32 *) &data[offset]);
    offset += 4;

    /* Calculate rr->rdlength after rdata's been built.  */

    /* Get the rdata, whose size may be self-determined or may be
       determined by the type.  */
    rr->rdata = (byte_array *) xcalloc(1, sizeof(byte_array));
    rr->rdata_uncompressed = (byte_array *) xcalloc(1, sizeof(byte_array));
    switch (rr->type) {
    case A:
        len = ntohs(*(const u16 *) &data[offset]);
        assert(len == 4); /* XXX: This will cause failure on bogus input  */ offset += 2;
        byte_array_catu32(rr->rdata, htonl(*(const u32 *) &data[offset]));
        /* No compression here.  */
        byte_array_cat(rr->rdata_uncompressed, rr->rdata);
        break;
    case NS:
    case CNAME:
    case TXT:
    case PTR:
        /* These cases are handled identically as far as marshalling
           and unmarshalling are concerned.  */
        // len = *(u8 *) &data[offset];
        len = ntohs(*(const u16 *) &data[offset]);
        // byte_array_catu16(rr->rdata, len);
        offset += 2;
        // byte_array_cats(rr->rdata, (const u8 *) &data[offset], len);
        // tmp = decompress2(&data[offset], base, len);
        byte_array_catu16(rr->rdata, len);
        
        tmp = decompress2(&data[offset], base);
        byte_array_cat(rr->rdata, tmp);
        // byte_array_cat(rr->rdata_uncompressed, tmp); 
        break;
    case MX:
        byte_array_catu16(rr->rdata, *(const u16 *) &data[offset]);
        offset += 2;
        len = *(const u8 *) &data[offset];
        offset += 2;
        byte_array_cats(rr->rdata, (const u8 *) &data[offset], len + 1);
        break;
    case SOA:
        len = *(const u8 *) &data[offset];
        byte_array_cats(rr->rdata, (const u8 *) &data[offset], len + 1);
        offset += len;

        len = *(const u8 *) &data[offset];
        byte_array_cats(rr->rdata, (const u8 *) &data[offset], len);
        offset += len;

        byte_array_catu32(rr->rdata, *(const u32 *) &data[offset]);
        offset += 4;

        byte_array_catu32(rr->rdata, *(const u32 *) &data[offset]);
        offset += 4;
        
        byte_array_catu32(rr->rdata, *(const u32 *) &data[offset]);
        offset += 4;
        
        byte_array_catu32(rr->rdata, *(const u32 *) &data[offset]);
        offset += 4;
        
        byte_array_catu32(rr->rdata, *(const u32 *) &data[offset]);
        offset += 4;
        break;
    case HINFO:
        UNIMPLEMENTED;
        break; 
    default:
        puts("""""""""""""""""");
        UNIMPLEMENTED;
        break;
    }

    printf("It's of type %d\n", rr->type);

    rr->rdlength = rr->rdata->len;

    retval = rr;

    *size = offset + retval->rdlength;

    if (tmp != NULL)
        byte_array_destroy(tmp);
   
    return retval;
}


int
packet_add_answer_from_data(packet *p, const u8 *data, int idx, const u8 *base)
{
    int retval;
    struct resource_record *rr;
    rr = build_rr_from_data(data, &retval, base);
    p->an[idx] = rr;
    // return rr->rdlength;
    return retval;
}


int
packet_add_authoritive_from_data(packet *p, const u8 *data, int idx, const u8 *base)
{
    int retval;
    struct resource_record *rr;
    rr = build_rr_from_data(data, &retval, base);
    p->ns[idx] = rr;
    // return rr->rdlength;
    return retval;
}

int
packet_add_additional_from_data(packet *p, const u8 *data, int idx, const u8 *base)
{
    int retval;
    struct resource_record *rr;
    rr = build_rr_from_data(data, &retval, base);
    p->ar[idx] = rr;
    // return rr->rdlength;
    return retval;
}


/* FIXME: This creates a fair amount of unnecessary redundancy.  As a result,
   there's excessive allocation, copying, and room for error.  We can
   probably do better.  */
struct resource_record *
record_to_resource_record(record *r, int *offset, ht *locations)
{
    struct resource_record *retval;

    retval = (struct resource_record *) xcalloc(1, sizeof(struct resource_record));
    retval->name = xstrdup(r->name);
    retval->type = r->type;
    retval->class = IN;
    retval->ttl = r->ttl;

    /* XXX: Are these enough?  */
    switch (r->type) {
    case A:
        retval->rdata = record_to_data_a(r);
        break;
    case NS:
        retval->rdata = record_to_data_ns(r, offset, locations);
        break;
    case MX:
        retval->rdata = record_to_data_mx(r, offset, locations);
        break;
    case CNAME:
        retval->rdata = record_to_data_cname(r, offset, locations);
        break;
    case TXT:
        retval->rdata = record_to_data_txt(r, offset, locations);
        break;
    case SOA:
        retval->rdata = record_to_data_soa(r);
        break;
    case PTR:
        retval->rdata = record_to_data_ptr(r);
        break;
    case HINFO:
        retval->rdata = record_to_data_hinfo(r);
        break;
    default:
        /* FIXME: deallocate memory allocated via a bogus query.  This could
           be exploited as a resource exhaustion denial of service attack.  */
        /* Or we could use a garbage collector.  */
        retval = NULL;
        break;
    }

    if (retval != NULL) 
        retval->rdlength = retval->rdata->len;

    return retval;
}

record *
resource_record_to_record(struct resource_record *rr)
{
    record *retval;
    retval = record_alloc();
    strncpy(retval->name, rr->name, 256);
    retval->ttl = rr->ttl;
    retval->type = rr->type;

    switch (rr->type) {
    case A:
        rr_to_record_a(retval, rr->rdata);
        break;
    case NS:
        // rr_to_record_ns(retval, rr->rdata_uncompressed);
        rr_to_record_ns(retval, rr->rdata);
        break;
    case MX:
        rr_to_record_mx(retval, rr->rdata);
        break;
    case CNAME:
        rr_to_record_cname(retval, rr->rdata);
        break;
    case TXT:
        rr_to_record_txt(retval, rr->rdata);
        break;
    case SOA:
        rr_to_record_soa(retval, rr->rdata);
        break;
    case PTR:
        rr_to_record_ptr(retval, rr->rdata);
        break;
    case HINFO:
        rr_to_record_hinfo(retval, rr->rdata);
        break;
    default:
        retval = NULL;
        break;
    }

    return retval;
}

byte_array *
record_to_data_a(record *r)
{
    byte_array *retval;

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    byte_array_catu32(retval, htonl(r->record.a));

    return retval;
}

void
rr_to_record_a(record *r, byte_array *rd)
{
    r->record.a = htonl(*(u32 *) rd->data);
}

byte_array *
record_to_data_ns(record *r, int *offset, ht *locations) 
{
    byte_array *retval;
    byte_array *tmp;
    
    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    
    tmp = name_to_bytes(r->record.ns, offset, locations);
    byte_array_cat(retval, tmp);

    return retval;
}

void
rr_to_record_ns(record *r, byte_array *rd)
{
    char *tmp;
    int size;
    tmp = question_arr_to_char_nd(rd->data, &size);
    if (size == 0) {
        ;/* XXX: Do something  */
    } else {
        ;
    }
    strncpy(r->record.ns, tmp, 256);
    xfree(tmp);
}

byte_array *
record_to_data_mx(record *r, int *offset, ht *locations) 
{
    byte_array *retval;
    byte_array *tmp;

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    byte_array_catu16(retval, r->record.mx.preference);
    tmp = name_to_bytes(r->record.mx.exchange, offset, locations);
    byte_array_cat(retval, tmp);

    return retval;
}

void
rr_to_record_mx(record *r, byte_array *rd)
{
    char *tmp;
    int did_compress;
    r->record.mx.preference = *(u16 *) rd->data;
    tmp = question_arr_to_char(rd->data + sizeof(u16), NULL, &did_compress);
    if (did_compress == 0) {
        ; /* XXX: do something  */
    } else {
        ;
    }
    strncpy(r->record.mx.exchange, tmp, 256);
    xfree(tmp);
}

byte_array *
record_to_data_cname(record *r, int *offset, ht *locations) 
{
    byte_array *retval;
    byte_array *tmp;

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    tmp = name_to_bytes(r->record.cname, offset, locations);
    byte_array_cat(retval, tmp);

    return retval;
}

void
rr_to_record_cname(record *r, byte_array *rd, int *offset, ht *locations)
{
    char *tmp;
    int did_compress;
    tmp = question_arr_to_char(rd->data, NULL, &did_compress);
    if (did_compress == 0) {
        ; /* XXX: do something  */
    } else {
        ;
    }
    strncpy(r->record.mx.exchange, tmp, 256);
    xfree(tmp);
}

byte_array *
record_to_data_txt(record *r)
{
    byte_array *retval;
    int len;

    /* From the RFC:  "<character-string> is a single length octet
       followed by that number of characters.  <character-string> is
       treated as binary information, and can be up to 256 characters
       in length (including the length octet).  */

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));

    len = strlen(r->record.txt);
    if (len > 255)
        len = 255;
    
    byte_array_catu8(retval, (u8) len);
    byte_array_cats(retval, r->record.txt, len);
    // byte_array_catu8(retval, '\0');
    
    return retval;
}

void
rr_to_record_txt(record *r, byte_array *rd)
{
    strncpy(r->record.txt, rd->data + sizeof(u8), 256);
}

byte_array *
record_to_data_soa(record *r)
{
    byte_array *retval;
    byte_array *tmp;
    
    retval = (byte_array *) xcalloc(1, sizeof(byte_array));

    tmp = name_to_bytes(r->record.soa.mname);
    byte_array_cat(retval, tmp);

    tmp = name_to_bytes(r->record.soa.rname);
    byte_array_cat(retval, tmp);

    byte_array_catu32(retval, htonl(r->record.soa.serial));

    byte_array_catu32(retval, htonl(r->record.soa.refresh));

    byte_array_catu32(retval, htonl(r->record.soa.retry));

    byte_array_catu32(retval, htonl(r->record.soa.expire));

    byte_array_catu32(retval, htonl(r->record.soa.minimum));

    return retval;
}

void
rr_to_record_soa(record *r __attribute__((unused)),
                 byte_array *rd __attribute__((unused)))
{
        
}

byte_array *
record_to_data_ptr(record *r) 
{
    byte_array *retval;
    byte_array *tmp;

    retval = (byte_array *) xcalloc(1, sizeof(byte_array));
    tmp = name_to_bytes(r->record.ptr);
    byte_array_cat(retval, tmp);

    return retval;
}

void
rr_to_record_ptr(record *r __attribute__((unused)),
                 byte_array *rd __attribute__((unused)))
{

}


byte_array *
record_to_data_hinfo(record *r __attribute__((unused))) 
{
    UNIMPLEMENTED;
    return NULL;
}

void
rr_to_record_hinfo(record *r __attribute__((unused)),
                   byte_array *rd __attribute__((unused)))
{
    UNIMPLEMENTED;
}

size_t
resource_record_size(struct resource_record *rr)
{
    size_t retval = 0;

    assert(rr->rdlength == rr->rdata->len);

    retval += 1 + strlen(rr->name) + 1; /* first byte, chars + byte per dot, nul terminating byte  */
    retval += 2; /* type  */
    retval += 2; /* class  */
    retval += 4; /* ttl  */
    retval += 2; /* rdlength  */
    retval += rr->rdlength; /* rdata  */

    return retval;
}
