/* Copyright (c) 2001, 2002 Omar Kilani, Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/query.c,v 1.3 2002/06/19 14:02:45 mawolf Exp $  */

/* Known issues:

   We only check return packets of <= 512 bytes.  Will need to deal with larger
   ones when we get around to adding TCP support.  */

/* Generalised query sending.  */

#ifndef _REENTRANT
#define _REENTRANT
#endif

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <assert.h>
#include <pthread.h>

#include "wpthread.h"
#include "socket.h"
#include "byte_array.h"
#include "protocol.h"
#include "query.h"

#include "xmalloc.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif

/* Send a query with a timeout.  */
packet *
query_send(packet *p, u32 ip, u16 port, int timeout)
{
    // char buf[20];
    struct in_addr foo;
    char *n, *n2;
    static pthread_mutex_t m = PTHREAD_MUTEX_INITIALIZER;
    
    wpthread_mutex_lock(&m);
    // XXX: fix this.  or at least slap a lock around it.  :(
    foo.s_addr = ip;
    n = inet_ntoa(foo);
    n2 = strdupa(n); // n2 = xstrdup(n);
    wpthread_mutex_unlock(&m);
    // memset(buf, '\0', sizeof(buf));
    // snprintf(buf, 20, "%d", ip);

    return query_send_char(p, n2, port, timeout);
}

packet *
query_send_char(packet *p, char *name, u16 port, int timeout)
{
    packet *retval;
    byte_array *input;
    byte_array *res;
    socket_t *s;
    socket_address_t sat;
    int len1, len2;
    
    sat.internet.address = name;
    sat.internet.port = port; // XXX: Should this be 53 or port?  Not sure.

    s = socket_sender(&sat, SOCKET_INTERNET, SOCKET_DATAGRAM);

    input = packet_convert_to_data(p);
    
    /* XXX: Should there be a timeout here?  */
    len1 = socket_send_timeout(s, input, 0, 0); 

    res = (byte_array *) xcalloc(1, sizeof(byte_array));
    len2 = socket_recv_timeout(s, res, 512, timeout, 0);
    if (len2 == 0) {
        puts("timeout");
        return NULL;
    }
    retval = packet_create_from_data(res);

    /* Cast to an int should be safe, since retval->size_in_bytes can't be 
       more than 512.  */
    printf("retval->size_in_bytes = %d, len2 = %d\n", retval->size_in_bytes,
           len2);
    assert((int) retval->size_in_bytes == len2);
    
    if (close(s->fd) == -1)
        abort();
    xfree(s);
    byte_array_destroy(input);
    byte_array_destroy(res);

    return retval;
}







