/* Copyright (c) 2001, 2002 Michael Wolf.  All rights reserved. 
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/record.h,v 1.16 2002/06/19 14:02:45 mawolf Exp $  */

/* A simplified model of what we'll be storing and retrieving, at least in
   the cache.  */


#ifndef RECORD_H
#define RECORD_H

#include "misc.h"

/* If we ever decide to support other classes - unlikely, but who knows? - we
   will need to expand this enum.  */
enum record_class { IN = 1 }; 

extern const char *record_class_names[];

enum record_type { A = 1, NS, MD, MF, CNAME, SOA, MB, MG, MR, NULL_EXP, WKS, PTR, HINFO, MINFO, MX, TXT };

extern const char *record_type_names[];

/* This needs to be cleaned up.  There also needs to be a good way to convert
   from a record to a packet, and back.  
   
   No pointers, since this is intended to be easily marshallable.  */
struct record {
    enum record_type type;
    char name[256]; /* Used as a key type; might not be necessary.  */
    union {
        u32 a;
        char ns[256]; /* FIXME: probably replace w/ byte_array  */
        u8 cname[256]; /* FIXME: probably replace w/ byte_array  */
        struct {
            char mname[256];
            char rname[256];
            u32 serial; 
            u32 refresh;
            u32 retry;
            u32 expire;
            u32 minimum;
        } soa;
        u8 ptr[256];
        struct {
            u16 preference;
            char exchange[256];
        } mx;
        char txt[256];
    } record;
    u32 ip; /* FIXME: this will need to go.  Use record.a instead.  */
    unsigned int expiry;
    int ttl;
    u64 timestamp;
    bool is_local;
};
typedef struct record record;

record *record_alloc(void);
record *record_alloc_dup(record *);
int record_compare(record *, record *);
int record_ttl_compare(record *, record *);
unsigned int record_expiry(record *);
void record_delete(record *);
void record_debug_dump(record *);

struct record_chain {
    char *name;
    /* This can go into a DB.  */
    record *records;
    int records_len;
};
/* XXX: get rid of the many `struct's in the src.  */
typedef struct record_chain record_chain; 

#define RECORD_CHAIN_INIT_TMP(x, y) \
    x = (struct record_chain *) alloca(sizeof(struct record_chain)); \
    x->records = NULL; \
    x->records_len = 0; \
    x->name = alloca(strlen(y) + 1); \
    strcpy(x->name, y);
struct record_chain *record_chain_init(char *);
int record_chain_add(struct record_chain *, struct record *);
int record_chain_add_chain(struct record_chain *, struct record_chain *);
struct record **record_chain_get(struct record_chain *, enum record_type);
void record_chain_destroy(struct record_chain *); /* XXX: Write function to deallocate record_chains.  */

#endif
