/* Copyright (c) 2001 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 * 
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 * 
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* It's important that everything that's done here is done in the correct 
   order.  To wit, if you consider dropping root and chrooting to the "atomic"
   actions, you can't do it.

   Rather, you need to find out the uid you'll want to switch to (in uid_t
   form), then chroot, then drop privs.  (That's because /etc/passwd or 
   similar probably won't be in the dir you chroot to.)  */
 
#ifndef _REENTRANT
#define _REENTRANT 
#endif 

#if HAVE_CONFIG_H  
#include "../config.h" 
#endif 

#include <string.h> 
#include <stdlib.h>
#include <stdio.h> 
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h> 
#include <assert.h>
#include <errno.h> 
 
#include <pthread.h>
 
#include "misc.h" 
#include "options.h" 
#include "security.h"

#include "xmalloc.h"  
#if WITH_DMALLOC 
#include <dmalloc.h> 
#endif
 
extern struct global_options globals;
 
void
drop_privs() 
{ 
    int do_chroot = 0;
    int do_setuid = 0;
    int ret1, ret2, ret3;
    struct passwd *pw = NULL;

    printf("We are uid %d and euid %d\n", getuid(), geteuid());


    if (globals.security.directory != NULL) {
        do_chroot = 1;
    } else {
        printf("Warning - not switching to a safe directory\n");
    }

    if (globals.security.user_name != NULL) {
        do_setuid = 1;
    } else {
        printf("Warning - root privileges not dropped\n");
    }

    if (do_setuid == 1) {
        errno = 0;
        pw = getpwnam(globals.security.user_name);
        if (pw == NULL) {
            perror("getpwnam() failed");
            if (errno == 0) {
                /* This is crap.  There should be a non-zero value for
                   errno here.  */
                printf("(probably due to trying to drop to a non-existent "
                       "user)\n");
            }
            abort();
        } 
        assert(!strcmp(pw->pw_name, globals.security.user_name));
    }
    
    if (do_chroot == 1) {
        ret1 = chdir(globals.security.directory);
        if (ret1 != 0) {
            perror("chdir() failed");
            abort();
        }
        ret2 = chroot(globals.security.directory);
        if (ret2 != 0) {
            perror("chroot() failed");
            abort();
        }
    }

    if (do_setuid == 1) {
        ret3 = setuid(pw->pw_uid);
        if (ret3 != 0) {
            perror("setuid() failed");
            abort();
        }
    }
}       

