/* Copyright (c) 2001 Omar Kilani.  All rights reserved.
 * Copyright (c) 2002 Michael Wolf.  All rights reserved. 
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef SOCKET_H
#define SOCKET_H

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "byte_array.h"

enum socket_domain { SOCKET_INTERNET = 0, SOCKET_LOCAL };
enum socket_type { SOCKET_STREAM = 0, SOCKET_DATAGRAM, SOCKET_SEQPACKET };
enum socket_job { SOCKET_LISTENER = 0, SOCKET_SENDER, SOCKET_CLIENT };
enum socket_blocking { SOCKET_BLOCKING = 0, SOCKET_NONBLOCKING };
/* maybe other attributes would be useful */
enum socket_close { SOCKET_READING = SHUT_RD, SOCKET_WRITING = SHUT_WR, 
                    SOCKET_BOTH = SHUT_RDWR };


typedef union socket_address_t {
    struct {
        int port;
        /* If address is nonnull, we use it, else we use IP.  */
        char *address;   
        unsigned int ip;
    } internet;
    struct {
        char *path;
    } local;
} socket_address_t;

typedef struct {
    int fd;
    enum socket_domain domain;
    enum socket_type type;
    enum socket_job job;
    enum socket_blocking blocking;
    union {
        struct sockaddr dummy;
        struct sockaddr_in internet;
        struct sockaddr_un local;
    } socket_address;
} socket_t;

/*@null@*/
socket_t *socket_listener(socket_address_t *addr, enum socket_domain domain,
                          enum socket_type type);
/*@null@*/
socket_t *socket_sender(socket_address_t *addr, enum socket_domain domain, 
                        enum socket_type type);
socket_t *socket_accept(socket_t *s);
int socket_close(socket_t *s, enum socket_close sc);

size_t socket_recv(socket_t *s, byte_array *ba, size_t maxlen);
size_t socket_recv_poll(socket_t *s, byte_array *ba, size_t maxlen);
size_t socket_recv_timeout(socket_t *s, byte_array *ba, size_t maxlen, 
                           int secs, int usecs);

size_t socket_send(socket_t *s, byte_array *ba);
size_t socket_send_poll(socket_t *s, byte_array *ba);
size_t socket_send_timeout(socket_t *s, byte_array *ba, int secs, int usecs);


/* attributes */
void socket_set_attribute_blocking(socket_t *, enum socket_blocking);

#endif
