/* Copyright (c) 2001 Michael Wolf.  All rights reserved.  
 *
 * This file is part of moodns.
 *
 * moodns is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * moodns is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with moodns; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Header: /cvsroot/moodns/moodns/src/st-test.c,v 1.5 2002/02/21 05:16:21 mawolf Exp $  */

/* Single-threaded test.  Useful mainly to debug stuff.  */

#if HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <pwd.h>

#include "wpthread.h"
#include "pool.h"

#include "misc.h"

#include "socket.h"
#include "options.h"
#include "log.h"
#include "cache.h"
#include "record.h"
#include "security.h"
#include "database.h"

#include "protocol.h"

#include "xmalloc.h"
#if WITH_DMALLOC
#include <dmalloc.h>
#endif

// #ifdef __FreeBSD
char *malloc_options = (char *) &"AJR";
// #endif

/* Set this to something greater than zero, and st-test will exit.  Useful with
   dmalloc.  */

// int expected_iters = 5;
int expected_iters = 0;

struct connection_wrappers {
    socket_t *tcp;
    socket_t *udp;

    pthread_mutex_t mutex;
    pthread_cond_t cv;
};


#include "handle_client.h"

database *d;
const char *MODULE_PATH = "libdatabase-ht.la";
record_chain *roots;

cache *c;

struct global_options globals;
int max(int, int);

int
main(int argc __attribute__((unused)), char **argv __attribute__((unused)))
{
    struct connection_wrappers *sh;
    socket_address_t sat;
    fd_set fullset;

    setenv("DMALLOC_OPTIONS", "debug=0x4f47d03,inter=1,log=logfile", 1); 

    srandom(time(NULL));

    initialise_roots();

    c = cache_init_st();

    options_init();

    sh = (struct connection_wrappers *) alloca(sizeof(struct connection_wrappers));
    memset(sh, 0, sizeof(*sh));
    sat.internet.port = globals.net.port;
    sat.internet.address = NULL;
    sh->tcp = socket_listener(&sat, SOCKET_INTERNET, SOCKET_STREAM);
    if (sh->tcp == NULL) 
        abort();
    sh->udp = socket_listener(&sat, SOCKET_INTERNET, SOCKET_DATAGRAM);
    if (sh->udp == NULL)
        abort();

    d = database_init(NULL);
    d->open_file(d, "../doc/data");
    d->load(d);

    while (1) {
        int s = 0;
        // fd_set ready;
        struct timeval timeout;

        if (!sh->tcp || !sh->udp) {
            printf("we're not talking to the network!\n");
            abort();
        }

        FD_ZERO(&fullset);
        FD_SET(sh->tcp->fd, &fullset);
        FD_SET(sh->udp->fd, &fullset);
        timeout.tv_sec = 60;
        timeout.tv_usec = 0;
        if ((s = select(max(sh->tcp->fd, sh->udp->fd) + 1, &fullset, NULL, NULL, &timeout)) < 0) {
            perror("select failed");
            continue;
        }

        if (s == 0) {
            printf("DEBUG: no data waiting.\n");
            continue;
        }

        if (FD_ISSET(sh->tcp->fd, &fullset)) {
            socket_t *client; // = xmalloc(sizeof(socket_t));
            // int n = sizeof(struct sockaddr_in);
            client = socket_accept(sh->tcp);
            // client->fd = accept(sh->tcp->fd, (struct sockaddr *) &client->remote, &n);
            // client->transport = SOCKET_TCP;
            handle_client(client);
        }
        
        if (FD_ISSET(sh->udp->fd, &fullset)) {
            socket_t *client; // = xcalloc(1, sizeof(socket_t));
            // int n = sizeof(struct sockaddr_in);
            // int x;
            // client->fd = sh->udp->fd;
            // client->type = SOCKET_LISTENER;
            // client->transport = SOCKET_UDP;
            client = socket_accept(sh->udp);
            handle_client(client);
        }
        if (expected_iters > 0 && expected_iters == get_count())
            break;
    }
                     

    return 0;
}

int 
max(int a, int b)
{
    if (a > b)
        return a;
    else 
        return b;
}
