/* 
 * Part of the pthread_utils package.  
 *
 * Copyright (c) 2001 Michael Wolf.  All rights reserved.  
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Header: /cvsroot/moodns/moodns/src/wpthread.h,v 1.2 2001/12/21 13:28:34 mawolf Exp $  */

#ifndef WPTHREAD_H
#define WPTHREAD_H

#include <pthread.h>

/* The w* wrappers are mostly identical to their w-less counterparts, except
   that they do the return value checking automagically.  It's handy that
   the pthread functions are almost consistent.  */

void wpthread_cond_init(pthread_cond_t *, const pthread_condattr_t *); 
void wpthread_mutex_init(pthread_mutex_t *, const pthread_mutexattr_t *);
void wpthread_create(pthread_t *, const pthread_attr_t *, 
		     void *(*)(void *), void *); 
void wpthread_join(pthread_t, void **); 
void wpthread_detach(pthread_t);
void wpthread_cond_wait(pthread_cond_t *, pthread_mutex_t *);
void wpthread_cond_signal(pthread_cond_t *);
void wpthread_mutex_lock(pthread_mutex_t *);
void wpthread_mutex_unlock(pthread_mutex_t *); 
void wpthread_cond_broadcast(pthread_cond_t *);
void wpthread_once(pthread_once_t *, void (*)(void));

/* These ones do not have the same calling conventions - we care about their
   return values beyond checking for errors.  What do we do if an argument to
   wpthread_equal() isn't an initialised pthread_t?  Clearly there should be
   some sort of error.  */
pthread_t wpthread_self(void);
int wpthread_equal(pthread_t, pthread_t);
int wpthread_mutex_trylock(pthread_mutex_t *);

#endif
